@@ -62,6 +62,8 @@
 #include "bgadvanced.h"
 #include "bgdialog.h"
 
+#include "kworkspace/screenpreviewwidget.h"
+
 #define NR_PREDEF_PATTERNS 6
 
 BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
@@ -150,9 +152,6 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
    // renderers
    if (m_numScreens > 1)
    {
-      m_renderer.resize(m_numScreens+2);
-      m_renderer.setAutoDelete(true);
-
       // Setup the merged-screen renderer
       KBackgroundRenderer * r = new KBackgroundRenderer(0, false, _config);
       m_renderer.insert( 0, r );
@@ -173,9 +172,6 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
    }
    else
    {
-      m_renderer.resize(1);
-      m_renderer.setAutoDelete(true);
-
       // set up the common desktop renderer
       KBackgroundRenderer * r = new KBackgroundRenderer(0, false, _config);
       m_renderer.insert(0, r);
@@ -210,6 +206,7 @@ BGDialog::BGDialog(QWidget* parent, const KSharedConfigPtr &_config)
 BGDialog::~BGDialog()
 {
    delete m_pGlobals;
+   qDeleteAll(m_renderer);
 }
 
 KBackgroundRenderer * BGDialog::eRenderer()
@@ -257,7 +254,7 @@ void BGDialog::load()
    m_pGlobals->readSettings();
    getEScreen();
 
-   for (unsigned screen = 0; screen < m_renderer.size(); ++screen)
+   for (int screen = 0; screen < m_renderer.size(); ++screen)
    {
       unsigned eScreen = screen>1 ? screen-2 : 0;
       m_renderer[screen]->load( eScreen, (screen>0), true );
@@ -289,7 +286,7 @@ void BGDialog::save()
    // depending on which are the real settings
    // they both share Screen[0] in the config file
 
-      for (unsigned screen = 0; screen < m_renderer.size(); ++screen)
+      for (int screen = 0; screen < m_renderer.size(); ++screen)
       {
          if (screen == 1 && !m_pGlobals->commonScreenBackground())
             continue;
@@ -416,8 +413,8 @@ void BGDialog::initUI()
 
    m_patterns = KBackgroundPattern::list();
    m_patterns.sort(); // Defined order
-   QStringList::Iterator it;
-   for (it=m_patterns.begin(); it != m_patterns.end(); ++it)
+   QStringList::const_iterator it;
+   for (it=m_patterns.constBegin(); it != m_patterns.constEnd(); ++it)
    {
       KBackgroundPattern pat(*it);
       if (pat.isAvailable())
@@ -460,7 +457,7 @@ void BGDialog::loadWallpaperFilesList() {
    //search for .desktop files before searching for images without .desktop files
    QStringList lst = m_pDirs->findAllResources("wallpaper", "*desktop", KStandardDirs::NoDuplicates);
    QStringList files;
-   for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
+   for (QStringList::ConstIterator it = lst.constBegin(); it != lst.constEnd(); ++it)
    {
       KDesktopFile fileConfig(*it);
       KConfigGroup cg = fileConfig.group("Wallpaper");
@@ -493,7 +490,7 @@ void BGDialog::loadWallpaperFilesList() {
 
    //now find any wallpapers that don't have a .desktop file
    lst = m_pDirs->findAllResources("wallpaper", "*", KStandardDirs::NoDuplicates);
-   for (QStringList::ConstIterator it = lst.begin(); it != lst.end(); ++it)
+   for (QStringList::ConstIterator it = lst.constBegin(); it != lst.constEnd(); ++it)
    {
       if ( !(*it).endsWith(".desktop") && files.filter(*it).empty() ) {
          // First try to see if we have a comment describing the image.  If we do
@@ -596,8 +593,8 @@ void BGDialog::slotWallpaperSelection()
 
    int j = m_urlWallpaperBox->currentIndex();
    QString uri;
-   for(QMap<QString,int>::ConstIterator it = m_wallpaper.begin();
-       it != m_wallpaper.end();
+   for(QMap<QString,int>::ConstIterator it = m_wallpaper.constBegin();
+       it != m_wallpaper.constEnd();
        ++it)
    {
       if (it.value() == j)
@@ -616,7 +613,7 @@ void BGDialog::slotWallpaperSelection()
       setWallpaper(dlg.selectedFile());
 
       int optionID = m_buttonGroupBackground->id(m_radioPicture);
-      m_buttonGroupBackground->setButton( optionID );
+      m_buttonGroupBackground->setSelected( optionID );
       slotWallpaperTypeChanged( optionID );
 
       emit changed(true);
@@ -653,7 +650,7 @@ void BGDialog::updateUI()
             m_comboWallpaperPos->setEnabled(false);
             m_lblWallpaperPos->setEnabled(false);
          }
-         m_buttonGroupBackground->setButton(
+         m_buttonGroupBackground->setSelected(
             m_buttonGroupBackground->id(m_radioNoPicture) );
       }
 
@@ -668,7 +665,7 @@ void BGDialog::updateUI()
             m_lblWallpaperPos->setEnabled(true);
          }
          setWallpaper(r->wallpaper());
-         m_buttonGroupBackground->setButton(
+         m_buttonGroupBackground->setSelected(
             m_buttonGroupBackground->id(m_radioPicture) );
       }
    }
@@ -683,7 +680,7 @@ void BGDialog::updateUI()
          m_comboWallpaperPos->setEnabled(true);
          m_lblWallpaperPos->setEnabled(true);
       }
-      m_buttonGroupBackground->setButton(
+      m_buttonGroupBackground->setSelected(
          m_buttonGroupBackground->id(m_radioSlideShow) );
    }
 
@@ -752,7 +749,7 @@ void BGDialog::updateUI()
        for (unsigned j = 0; j < m_numScreens; ++j)
        {
           m_renderer[j+2]->stop();
-          m_renderer[j+2]->setPreview( m_pMonitorArrangement->monitor(j)->size() );
+          m_renderer[j+2]->setPreview( m_pMonitorArrangement->monitor(j)->previewRect().size() );
           m_renderer[j+2]->start(true);
        }
     }
@@ -779,11 +776,11 @@ void BGDialog::slotPreviewDone(int screen_done)
    else if ( m_eScreen == 1 )
    {
       for (unsigned i = 0; i < m_pMonitorArrangement->numMonitors(); ++i)
-         m_pMonitorArrangement->monitor(i)->setPixmap(pm);
+         m_pMonitorArrangement->monitor(i)->setPreview(pm);
    }
    else
    {
-      m_pMonitorArrangement->monitor(screen_done)->setPixmap(pm);
+      m_pMonitorArrangement->monitor(screen_done)->setPreview(pm);
    }
 }
 
@@ -792,7 +789,7 @@ void BGDialog::slotImageDropped(const QString &uri)
    setWallpaper(uri);
 
    int optionID = m_buttonGroupBackground->id(m_radioPicture);
-   m_buttonGroupBackground->setButton( optionID );
+   m_buttonGroupBackground->setSelected( optionID );
    slotWallpaperTypeChanged( optionID );
 }
 
@@ -860,8 +857,8 @@ void BGDialog::slotWallpaperTypeChanged(int i)
 
       int j = m_urlWallpaperBox->currentIndex();
       QString path;
-      for(QMap<QString,int>::ConstIterator it = m_wallpaper.begin();
-          it != m_wallpaper.end();
+      for(QMap<QString,int>::ConstIterator it = m_wallpaper.constBegin();
+          it != m_wallpaper.constEnd();
           ++it)
       {
          if (it.value() == j)
@@ -1044,7 +1041,7 @@ void BGDialog::slotSelectScreen(int screen)
       emit changed(true);
    else
    {
-      for (unsigned i = 0; i < m_renderer.size(); ++i)
+      for (int i = 0; i < m_renderer.size(); ++i)
       {
          if ( m_renderer[i]->isActive() )
             m_renderer[i]->stop();
@@ -1162,7 +1159,7 @@ void BGDialog::slotBlendReverse(bool b)
 
 void BGDialog::desktopResized()
 {
-   for (unsigned j = 0; j < m_renderer.size(); ++j )
+   for (int j = 0; j < m_renderer.size(); ++j )
    {
       KBackgroundRenderer * r = m_renderer[j];
       if( r->isActive())
