@@ -238,7 +238,7 @@ void CJobRunner::getAssociatedUrls(const KUrl &url, KUrl::List &list, bool afmAn
 
     if(-1==dotPos) // Hmm, no extension - check anyway...
         check=true;
-    else           // Cool, got an extension - see if its a Type1 font...
+    else           // Cool, got an extension - see if it is a Type1 font...
     {
         ext=ext.mid(dotPos+1);
         check=0==ext.compare("pfa", Qt::CaseInsensitive) ||
@@ -247,8 +247,8 @@ void CJobRunner::getAssociatedUrls(const KUrl &url, KUrl::List &list, bool afmAn
 
     if(check)
     {
-        const char *afm[]={"afm", "AFM", "Afm", NULL},
-                   *pfm[]={"pfm", "PFM", "Pfm", NULL};
+        static const char * const afm[]={"afm", "AFM", "Afm", NULL};
+        static const char * const pfm[]={"pfm", "PFM", "Pfm", NULL};
         bool       gotAfm(false),
                    localFile(url.isLocalFile());
         int        e;
@@ -260,7 +260,7 @@ void CJobRunner::getAssociatedUrls(const KUrl &url, KUrl::List &list, bool afmAn
 
             statUrl.setPath(Misc::changeExt(url.path(), afm[e]));
 
-            if(localFile ? Misc::fExists(statUrl.path()) : KIO::NetAccess::stat(statUrl, uds, widget))
+            if(localFile ? Misc::fExists(statUrl.toLocalFile()) : KIO::NetAccess::stat(statUrl, uds, widget))
             {
                 list.append(statUrl);
                 gotAfm=true;
