@@ -17,12 +17,15 @@
 */
 
 #include "actions.h"
-#include "windows.h"
+#include "windows_handler.h"
+#include "windows_helper/window_selection_list.h"
 
 #include <KDE/KConfigGroup>
 
+#ifdef Q_WS_X11
 // Has to be behind all qt related stuff. Else the build fails miserably
 #include <X11/X.h>
+#endif
 
 namespace KHotKeys {
 
