@@ -16,12 +16,16 @@
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 */
-
-#include "conditions.h"
-
-#include <KDE/KConfigGroup>
 #include <KDE/KDebug>
 
+#include "conditions/existing_window_condition.h"
+
+#include "windows_helper/window_selection_list.h"
+
+#include "windows_handler.h"
+#include "khotkeysglobal.h"
+
+
 namespace KHotKeys {
 
 Existing_window_condition::Existing_window_condition( KConfigGroup& cfg_P, Condition_list_base* parent_P )
@@ -33,17 +37,59 @@ Existing_window_condition::Existing_window_condition( KConfigGroup& cfg_P, Condi
     set_match();
     }
 
+
+Existing_window_condition::Existing_window_condition( Windowdef_list* window_P,
+    Condition_list_base* parent_P )
+    : Condition( parent_P ), _window( window_P ), is_match( false )
+    {
+    init();
+    set_match();
+    }
+
+
+Existing_window_condition::~Existing_window_condition()
+    {
+    disconnect( windows_handler, NULL, this, NULL );
+    delete _window;
+    }
+
+
+void Existing_window_condition::cfg_write( KConfigGroup& cfg_P ) const
+    {
+    kDebug() << description();
+    base::cfg_write( cfg_P );
+    KConfigGroup windowConfig( cfg_P.config(), cfg_P.name() + "Window" );
+    window()->cfg_write( windowConfig );
+    cfg_P.writeEntry( "Type", "EXISTING_WINDOW" ); // overwrites value set in base::cfg_write()
+    }
+
+
+Existing_window_condition* Existing_window_condition::copy() const
+    {
+    kDebug();
+    return new Existing_window_condition(window()->copy());
+    }
+
+
+const QString Existing_window_condition::description() const
+    {
+    return i18n( "Existing window: " ) + window()->comment();
+    }
+
+
 void Existing_window_condition::init()
     {
     connect( windows_handler, SIGNAL( window_added( WId )), this, SLOT( window_added( WId )));
     connect( windows_handler, SIGNAL( window_removed( WId )), this, SLOT( window_removed( WId )));
     }
 
+
 bool Existing_window_condition::match() const
     {
     return is_match;
     }
 
+
 void Existing_window_condition::set_match( WId w_P )
     {
     if( w_P != None && !is_match )
@@ -54,38 +100,29 @@ void Existing_window_condition::set_match( WId w_P )
     updated();
     }
 
-void Existing_window_condition::cfg_write( KConfigGroup& cfg_P ) const
+
+const Windowdef_list* Existing_window_condition::window() const
     {
-    base::cfg_write( cfg_P );
-    KConfigGroup windowConfig( cfg_P.config(), cfg_P.name() + "Window" );
-    window()->cfg_write( windowConfig );
-    cfg_P.writeEntry( "Type", "EXISTING_WINDOW" ); // overwrites value set in base::cfg_write()
+    return _window;
     }
 
-Existing_window_condition* Existing_window_condition::copy( Condition_list_base* parent_P ) const
+
+Windowdef_list* Existing_window_condition::window()
     {
-    return new Existing_window_condition( window()->copy(), parent_P );
+    return _window;
     }
 
-const QString Existing_window_condition::description() const
-    {
-    return i18n( "Existing window: " ) + window()->comment();
-    }
 
 void Existing_window_condition::window_added( WId w_P )
     {
     set_match( w_P );
     }
 
+
 void Existing_window_condition::window_removed( WId )
     {
     set_match();
     }
 
-Existing_window_condition::~Existing_window_condition()
-    {
-    disconnect( windows_handler, NULL, this, NULL );
-    delete _window;
-    }
 
 } // namespace KHotKeys
