@@ -1,3 +1,4 @@
 #! /usr/bin/env bash
-#$EXTRACTRC `find . -name "*.ui"` >> rc.cpp || exit 11
+$EXTRACTRC kcfg/*.kcfg >> rc.cpp
 $XGETTEXT `find . -name \*.cpp -o -name \*.cc` -o $podir/krunner.pot
+rm -f rc.cpp
