@@ -58,8 +58,6 @@ public:
     bool hasCompositeManager() const;
 
 public Q_SLOTS:
-    //void showWindowList();
-
     void logout();
     void logoutWithoutConfirmation();
     void haltWithoutConfirmation();
@@ -71,6 +69,8 @@ public Q_SLOTS:
 
     /** Show taskmanager */
     Q_SCRIPTABLE void showTaskManager();
+    /** Show taskmanager, filtering by the given string */
+    Q_SCRIPTABLE void showTaskManagerWithFilter(const QString &filterText);
 
     /** Display the interface */
     Q_SCRIPTABLE void display();
@@ -96,7 +96,7 @@ private slots:
     void cleanUp();
 
 private:
-    KRunnerApp(Display *display, Qt::HANDLE visual = 0, Qt::HANDLE colormap = 0);
+    KRunnerApp();
     void initialize();
 
     Plasma::RunnerManager *m_runnerManager;
