@@ -45,6 +45,7 @@
 #include "LogFile.h"
 #include "MultiMeter.h"
 #include "ProcessController.h"
+#include "ProcessTable.h"
 #include "SensorLogger.h"
 #include "WorkSheet.h"
 #include "WorkSheetSettings.h"
@@ -55,20 +56,17 @@ WorkSheet::WorkSheet( QWidget *parent )
   mGridLayout = 0;
   mRows = mColumns = 0;
   mDisplayList = 0;
-  mFileName = "";
-  mLocalProcessController = NULL;
-
+  setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
   setAcceptDrops( true );
 }
 
-WorkSheet::WorkSheet( uint rows, uint columns, uint interval, QWidget* parent )
+WorkSheet::WorkSheet( uint rows, uint columns, float interval, QWidget* parent )
   : QWidget( parent)
 {
   mRows = mColumns = 0;
   mGridLayout = 0;
   mDisplayList = 0;
   setUpdateInterval( interval );
-  mFileName = "";
 
   createGrid( rows, columns );
 
@@ -84,6 +82,12 @@ WorkSheet::WorkSheet( uint rows, uint columns, uint interval, QWidget* parent )
 
 WorkSheet::~WorkSheet()
 {
+  for ( uint r = 0; r < mRows; ++r ) {
+    for ( uint c = 0; c < mColumns; ++c )
+      delete mDisplayList[ r ][ c ];
+    delete mDisplayList[ r ];
+  }
+  delete [] mDisplayList;
 }
 
 bool WorkSheet::load( const QString &fileName )
@@ -113,9 +117,9 @@ bool WorkSheet::load( const QString &fileName )
 
   // Check for proper size.
   QDomElement element = doc.documentElement();
-  unsigned int interval = element.attribute( "interval", "2" ).toUInt();
-  if( interval  <1 || interval > 900 )  //make sure the interval is fairly sane
-    interval = 2;
+  float interval = element.attribute( "interval", "0.5" ).toFloat();
+  if( interval  <0.01 || interval > 100000 )  //make sure the interval is fairly sane
+    interval = 0.5;
 
   setUpdateInterval(interval);
 
@@ -173,7 +177,7 @@ bool WorkSheet::load( const QString &fileName )
     for ( uint c = 0; c < mColumns; ++c )
       if ( !mDisplayList[ r ][ c ] )
         replaceDisplay( r, c );
-
+  mFullFileName = fileName;
   return true;
 }
 
@@ -291,6 +295,11 @@ void WorkSheet::setFileName( const QString &fileName )
   mFileName = fileName;
 }
 
+QString WorkSheet::fullFileName() const
+{
+  return mFullFileName;
+}
+
 QString WorkSheet::fileName() const
 {
   return mFileName;
@@ -311,18 +320,60 @@ QString WorkSheet::title() const {
   return mTitle;
 }
 
+KSGRD::SensorDisplay* WorkSheet::insertDisplay( DisplayType displayType, uint row, uint column)
+{
+  KSGRD::SensorDisplay* newDisplay = 0;
+  switch(displayType) {
+    case DisplayDummy: 
+      newDisplay = new DummyDisplay( this, &mSharedSettings );
+      break;
+    case DisplayFancyPlotter:
+      newDisplay = new FancyPlotter( this, NULL, &mSharedSettings );
+      break;
+    case DisplayMultiMeter:
+      newDisplay = new MultiMeter( this, NULL, &mSharedSettings);
+      break;
+    case DisplayDancingBars: 
+      newDisplay = new DancingBars( this, NULL, &mSharedSettings);
+      break;
+    case DisplaySensorLogger:
+      newDisplay = new SensorLogger( this, NULL, &mSharedSettings);
+      break;
+    case DisplayListView:
+      newDisplay = new ListView( this, NULL, &mSharedSettings);
+      break;
+    case DisplayLogFile:
+      newDisplay = new LogFile( this, NULL, &mSharedSettings );
+      break;
+    case DisplayProcessControllerRemote:
+      newDisplay = new ProcessController( this);
+      break;
+    case DisplayProcessControllerLocal:
+	Q_ASSERT(sLocalProcessController);
+	newDisplay = sLocalProcessController;
+	break;
+    default:
+      Q_ASSERT(false);
+      return NULL;
+  }
+  newDisplay->applyStyle();
+  connect(&mTimer, SIGNAL( timeout()), newDisplay, SLOT( timerTick()));
+  replaceDisplay( row, column, newDisplay );
+  return newDisplay;
+}
+
 KSGRD::SensorDisplay *WorkSheet::addDisplay( const QString &hostName,
                                              const QString &sensorName,
                                              const QString &sensorType,
                                              const QString& sensorDescr,
                                              uint row, uint column )
 {
-
+  KSGRD::SensorDisplay* display = mDisplayList[ row ][ column ];
   /* If the by 'row' and 'column' specified display is a QGroupBox dummy
    * display we replace the widget. Otherwise we just try to add
    * the new sensor to an existing display. */
-  if ( mDisplayList[ row ][ column ]->metaObject()->className() == QByteArray( "DummyDisplay" ) ) {
-    KSGRD::SensorDisplay* newDisplay = 0;
+  if ( display->metaObject()->className() == QByteArray( "DummyDisplay" ) ) {
+    DisplayType displayType = DisplayDummy;
     /* If the sensor type is supported by more than one display
      * type we popup a menu so the user can select what display is
      * wanted. */
@@ -334,49 +385,41 @@ KSGRD::SensorDisplay *WorkSheet::addDisplay( const QString &hostName,
       QAction *a3 = pm.addAction( i18n( "&Bar graph" ) );
       QAction *a4 = pm.addAction( i18n( "Log to a &file" ) );
       QAction *execed = pm.exec( QCursor::pos() );
-      if (execed == a1) {
-        newDisplay = new FancyPlotter( this, sensorDescr, &mSharedSettings );
-      }
-      else if (execed == a2) {
-        newDisplay = new MultiMeter( this, sensorDescr, &mSharedSettings);
-      }
-      else if (execed == a3) {
-        newDisplay = new DancingBars( this, sensorDescr, &mSharedSettings);
-      }
-      else if (execed == a4) {
-        newDisplay = new SensorLogger( this, sensorDescr, &mSharedSettings);
-      }
-      else {
+      if (execed == a1)
+  	displayType = DisplayFancyPlotter;
+      else if (execed == a2)
+	displayType = DisplayMultiMeter;
+      else if (execed == a3)
+	displayType = DisplayDancingBars;
+      else if (execed == a4)
+	displayType = DisplaySensorLogger;
+      else 
         return 0;
-      }
     } else if ( sensorType == "listview" ) {
-      newDisplay = new ListView( this, sensorDescr, &mSharedSettings);
+      displayType = DisplayListView;
     }
     else if ( sensorType == "logfile" ) {
-      newDisplay = new LogFile( this, sensorDescr, &mSharedSettings );
+      displayType = DisplayLogFile;
     }
     else if ( sensorType == "sensorlogger" ) {
-      newDisplay = new SensorLogger( this, sensorDescr, &mSharedSettings );
+      displayType = DisplaySensorLogger;
     }
     else if ( sensorType == "table" ) {
-      if(!mLocalProcessController && (hostName.isEmpty() || hostName == "localhost")) {
-        mLocalProcessController = new ProcessController( this, sensorDescr, &mSharedSettings);
-	newDisplay = mLocalProcessController;
-      } else 
-        newDisplay = new ProcessController( this, sensorDescr, &mSharedSettings);
+      if(hostName.isEmpty() || hostName == "localhost")
+        displayType = DisplayProcessControllerLocal;
+      else
+        displayType = DisplayProcessControllerRemote;
     }
     else {
       kDebug(1215) << "Unknown sensor type: " <<  sensorType;
       return 0;
     }
-
-    connect(&mTimer, SIGNAL( timeout()), newDisplay, SLOT( timerTick()));
-    replaceDisplay( row, column, newDisplay );
+    display = insertDisplay(displayType, row, column);
   }
 
-  mDisplayList[ row ][ column ]->addSensor( hostName, sensorName, sensorType, sensorDescr );
+  display->addSensor( hostName, sensorName, sensorType, sensorDescr );
 
-  return ((KSGRD::SensorDisplay*)mDisplayList[ row ][ column ] );
+  return display;
 }
 
 void WorkSheet::settings()
@@ -418,20 +461,46 @@ void WorkSheet::applyStyle()
       mDisplayList[ r ][ c ]->applyStyle();
 }
 
-void WorkSheet::dragEnterEvent( QDragEnterEvent *event )
+void WorkSheet::dragEnterEvent( QDragEnterEvent* event) 
 {
-  if ( !event->mimeData()->hasText() )
+  if ( !event->mimeData()->hasFormat("application/x-ksysguard") )
     return;
+  event->accept();
+}
+void WorkSheet::dragMoveEvent( QDragMoveEvent *event )
+{
 
-  event->acceptProposedAction();
+  /* Find the sensor display that is supposed to get the drop
+   * event and replace or add sensor. */
+  const QPoint globalPos = mapToGlobal( event->pos() );
+  for ( uint r = 0; r < mRows; ++r ) {
+    for ( uint c = 0; c < mColumns; ++c ) {
+      const QSize displaySize = mDisplayList[ r ][ c ]->size();
+
+      const QPoint displayPoint( displaySize.width(), displaySize.height() );
+
+      const QRect widgetRect = QRect( mDisplayList[ r ][ c ]->mapToGlobal( QPoint( 0, 0 ) ),
+                                      mDisplayList[ r ][ c ]->mapToGlobal( displayPoint ) );
+
+
+      if ( widgetRect.contains( globalPos ) ) {
+        QByteArray widgetType = mDisplayList[ r ][ c ]->metaObject()->className();
+        if(widgetType == "MultiMeter" || widgetType == "ProcessController" || widgetType == "table")
+		event->ignore(widgetRect);
+	else if(widgetType != "Dummy")
+  		event->accept(widgetRect);
+        return;
+      }
+    }
+  }
 }
 
 void WorkSheet::dropEvent( QDropEvent *event )
 {
-  if ( !event->mimeData()->hasText() )
+  if ( !event->mimeData()->hasFormat("application/x-ksysguard") )
     return;
 
-  const QString dragObject = event->mimeData()->text();
+  const QString dragObject = QString::fromUtf8(event->mimeData()->data("application/x-ksysguard"));
 
   // The host name, sensor name and type are separated by a ' '.
   QStringList parts = dragObject.split( ' ');
@@ -467,7 +536,7 @@ void WorkSheet::dropEvent( QDropEvent *event )
 
 QSize WorkSheet::sizeHint() const
 {
-  return QSize( 200,150 );
+  return QSize( 800,600 );
 }
 
 bool WorkSheet::event( QEvent *e )
@@ -490,40 +559,38 @@ bool WorkSheet::event( QEvent *e )
 bool WorkSheet::replaceDisplay( uint row, uint column, QDomElement& element )
 {
   QString classType = element.attribute( "class" );
-  KSGRD::SensorDisplay* newDisplay = 0;
-
+  QString hostName = element.attribute( "hostName" );
+  DisplayType displayType = DisplayDummy;
+  KSGRD::SensorDisplay* newDisplay;
 
   if ( classType == "FancyPlotter" )
-    newDisplay = new FancyPlotter( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplayFancyPlotter;
   else if ( classType == "MultiMeter" )
-    newDisplay = new MultiMeter( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplayMultiMeter;
   else if ( classType == "DancingBars" )
-    newDisplay = new DancingBars( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplayDancingBars;
   else if ( classType == "ListView" )
-    newDisplay = new ListView( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplayListView;
   else if ( classType == "LogFile" )
-    newDisplay = new LogFile( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplayLogFile;
   else if ( classType == "SensorLogger" )
-    newDisplay = new SensorLogger( 0, i18n("Dummy"), &mSharedSettings );
+    displayType = DisplaySensorLogger;
   else if ( classType == "ProcessController" ) {
-    if(!mLocalProcessController) {
-      kDebug() << "Found process controller"; 
-      mLocalProcessController = new ProcessController( 0, i18n("Dummy"), &mSharedSettings);
-      newDisplay = mLocalProcessController;
-    }
+    if(hostName.isEmpty() || hostName == "localhost")
+      displayType = DisplayProcessControllerLocal;
+    else
+      displayType = DisplayProcessControllerRemote;
   } else {
     kDebug(1215) << "Unknown class " <<  classType;
     return false;
   }
 
-  connect(&mTimer, SIGNAL( timeout()), newDisplay, SLOT( timerTick()));
+  newDisplay = insertDisplay(displayType, row, column);
 
   // load display specific settings
   if ( !newDisplay->restoreSettings( element ) )
     return false;
 
-  replaceDisplay( row, column, newDisplay );
-
   return true;
 }
 
@@ -531,9 +598,7 @@ bool WorkSheet::replaceDisplay( uint row, uint column, QDomElement& element )
 void WorkSheet::replaceDisplay( uint row, uint column, KSGRD::SensorDisplay* newDisplay )
 {
   // remove the old display && sensor frame at this location
-  if ( mDisplayList[ row ][ column ] ) {
-    if(mDisplayList[ row ][ column ] == mLocalProcessController) 
-      mLocalProcessController = NULL;
+  if ( mDisplayList[ row ][ column ] && mDisplayList[ row ][ column ] != sLocalProcessController)  {
     if ( qstrcmp( mDisplayList[ row ][ column ]->parent()->metaObject()->className(), "SensorFrame" ) == 0 ) {
       delete mDisplayList[ row ][ column ]->parent(); // destroys the child (display) as well
     } else {
@@ -560,8 +625,6 @@ void WorkSheet::replaceDisplay( uint row, uint column, KSGRD::SensorDisplay* new
   if ( isVisible() ) {
     mDisplayList[ row ][ column ]->show();
   }
-
-  setMinimumSize(sizeHint());
 }
 
 void WorkSheet::removeDisplay( KSGRD::SensorDisplay *display )
@@ -569,9 +632,6 @@ void WorkSheet::removeDisplay( KSGRD::SensorDisplay *display )
   if ( !display )
     return;
   
-  if(display == mLocalProcessController) 
-    mLocalProcessController = NULL;
-
   for ( uint r = 0; r < mRows; ++r )
     for ( uint c = 0; c < mColumns; ++c )
       if ( mDisplayList[ r ][ c ] == display ) {
@@ -719,7 +779,7 @@ void WorkSheet::changeEvent( QEvent * event ) {
   }
 }
 
-void WorkSheet::setUpdateInterval( unsigned int secs)
+void WorkSheet::setUpdateInterval( float secs)
 {
   if(secs == 0)
     mTimer.stop();
@@ -728,10 +788,10 @@ void WorkSheet::setUpdateInterval( unsigned int secs)
     mTimer.start();
   }
 }
-int WorkSheet::updateInterval() const
+float WorkSheet::updateInterval() const
 {
   if(mTimer.isActive())
-    return mTimer.interval()/1000;
+    return mTimer.interval()/1000.0;
   else
     return 0;
 }
