@@ -17,15 +17,13 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    KSysGuard is currently maintained by Chris Schlaeger <cs@kde.org>. Please do
-    not commit any changes without consulting me first. Thanks!
-
 */
 
 #ifndef KSG_WORKSPACE_H
 #define KSG_WORKSPACE_H
 
 #include <KTabWidget>
+#include <kdirwatch.h>
 
 class KConfig;
 class KUrl;
@@ -46,7 +44,6 @@ class Workspace : public KTabWidget
     bool saveOnQuit();
 
     bool restoreWorkSheet( const QString &fileName, bool switchToTab = true);
-    void removeWorkSheet( const QString &fileName );
     QList<WorkSheet *> getWorkSheets() const { return mSheetList; }
 
   public Q_SLOTS:
@@ -57,8 +54,10 @@ class Workspace : public KTabWidget
     void exportWorkSheet();
     void exportWorkSheet( WorkSheet *sheet );
     void removeWorkSheet();
+    void removeWorkSheet( const QString &fileName );
     void removeAllWorkSheets();
     void getHotNewWorksheet();
+    void uploadHotNewWorksheet();
     void cut();
     void copy();
     void paste();
@@ -76,6 +75,7 @@ class Workspace : public KTabWidget
     QList<WorkSheet *> mSheetList;
 
     QString makeNameForNewSheet() const;
+    KDirWatch mDirWatch;
 };
 
 #endif
