@@ -59,6 +59,7 @@ class TopLevel : public KXmlGuiWindow, public KSGRD::SensorClient
     Q_SCRIPTABLE QStringList listHosts();
     Q_SCRIPTABLE QStringList listSensors( const QString &hostName );
 
+
   protected:
     virtual bool event( QEvent* );
     virtual void timerEvent( QTimerEvent* );
@@ -68,12 +69,10 @@ class TopLevel : public KXmlGuiWindow, public KSGRD::SensorClient
     void connectHost();
     void disconnectHost();
     void updateStatusBar();
-    void editToolbars();
-    void slotNewToolbarConfig();
     void currentTabChanged(int index);
 
   private:
-    void setSwapInfo( long, long, const QString& );
+    void setSwapInfo( qlonglong, qlonglong, const QString& );
     void changeEvent( QEvent * event );
     void retranslateUi();
 
@@ -94,7 +93,7 @@ class TopLevel : public KXmlGuiWindow, public KSGRD::SensorClient
     QAction *mHotNewWorksheetAction;
     QAction *mQuitAction;
     QAction *mConfigureSheetAction;
-
+    QAction *mHotNewWorksheetUploadAction;
     QLabel *sbProcessCount;
     QLabel *sbCpuStat;
     QLabel *sbMemTotal;
