@@ -411,7 +411,7 @@ void Workspace::performCompositing()
           // TODO I think effects->transformWindowDamage() doesn't need to be called here,
           // pre-paint will extend painted window areas as necessary.
         repaints_region |= c->repaints().translated( c->pos());
-        c->resetRepaints( c->rect());
+        c->resetRepaints( c->decorationRect());
         }
     QRegion repaints = repaints_region;
     // clear all repaints, so that post-pass can add repaints for the next repaint
@@ -958,6 +958,12 @@ bool Client::shouldUnredirect() const
     return false;
     }
 
+void Client::addRepaintFull()
+    {
+    repaints_region = decorationRect();
+    workspace()->checkCompositeTimer();
+    }
+
 //****************************************
 // Unmanaged
 //****************************************
@@ -997,4 +1003,10 @@ bool Deleted::shouldUnredirect() const
     return false;
     }
 
+void Deleted::addRepaintFull()
+    {
+    repaints_region = decorationRect();
+    workspace()->checkCompositeTimer();
+    }
+
 } // namespace
