@@ -55,6 +55,7 @@ class KDecorationPreview
         void setTempBorderSize(KDecorationPlugins* plugin, KDecorationDefines::BorderSize size);
         void setTempButtons(KDecorationPlugins* plugin, bool customEnabled, const QString &left, const QString &right);
     protected:
+        virtual void paintEvent( QPaintEvent* );
         virtual void resizeEvent( QResizeEvent* );
     private:
         void positionPreviews();
@@ -113,10 +114,7 @@ class KDecorationPreviewBridge
         virtual Qt::WFlags initialWFlags() const;
         virtual void grabXServer( bool grab );
 
-        virtual void repaintShadow();
         virtual bool compositingActive() const;
-        virtual bool shadowsActive() const;
-        virtual double opacity() const;
     private:
         KDecorationPreview* preview;
         bool active;
