@@ -35,6 +35,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "client.h"
 #include "compositingprefs.h"
 
+#include <kephal/screens.h>
+
 #endif
 
 namespace KWin
@@ -43,8 +45,8 @@ namespace KWin
 #ifndef KCMRULES
 
 Options::Options()
-    :   electric_borders( 0 ),
-        electric_border_delay(0)
+    : electric_borders( 0 )
+    , electric_border_delay( 0 )
     {
     updateSettings();
     }
@@ -59,7 +61,7 @@ unsigned long Options::updateSettings()
     unsigned long changed = 0;
     changed |= KDecorationOptions::updateSettings( _config.data() ); // read decoration settings
 
-    KConfigGroup config(_config, "Windows");
+    KConfigGroup config( _config, "Windows" );
     moveMode = stringToMoveResizeMode( config.readEntry("MoveMode", "Opaque" ));
     resizeMode = stringToMoveResizeMode( config.readEntry("ResizeMode", "Opaque" ));
     show_geometry_tip = config.readEntry("GeometryTip", false);
@@ -79,7 +81,7 @@ unsigned long Options::updateSettings()
     altTabStyle = KDE; // what a default :-)
     if ( val == "CDE" )
         altTabStyle = CDE;
-        
+
     separateScreenFocus = config.readEntry( "SeparateScreenFocus", false );
     activeMouseScreen = config.readEntry( "ActiveMouseScreen", focusPolicy != ClickToFocus );
 
@@ -92,16 +94,15 @@ unsigned long Options::updateSettings()
     if( !focusPolicyIsReasonable()) // #48786, comments #7 and later
         focusStealingPreventionLevel = 0;
 
-    KConfigGroup gWindowsConfig( _config, "Windows"); // from kdeglobals
-    xineramaEnabled = gWindowsConfig.readEntry ("XineramaEnabled", true);
-    xineramaPlacementEnabled = gWindowsConfig.readEntry ("XineramaPlacementEnabled", true);
-    xineramaMovementEnabled = gWindowsConfig.readEntry ("XineramaMovementEnabled", true);
-    xineramaMaximizeEnabled = gWindowsConfig.readEntry ("XineramaMaximizeEnabled", true);
-    xineramaFullscreenEnabled = gWindowsConfig.readEntry ("XineramaFullscreenEnabled", true);
+    xineramaEnabled = config.readEntry ("XineramaEnabled", true);
+    xineramaPlacementEnabled = config.readEntry ("XineramaPlacementEnabled", true);
+    xineramaMovementEnabled = config.readEntry ("XineramaMovementEnabled", true);
+    xineramaMaximizeEnabled = config.readEntry ("XineramaMaximizeEnabled", true);
+    xineramaFullscreenEnabled = config.readEntry ("XineramaFullscreenEnabled", true);
 
     placement = Placement::policyFromString( config.readEntry("Placement"), true );
     xineramaPlacementScreen = qBound( -1, config.readEntry( "XineramaPlacementScreen", -1 ),
-        qApp->desktop()->numScreens() - 1 );
+        Kephal::ScreenUtils::numScreens() - 1 );
 
     if( focusPolicy == ClickToFocus )
         {
@@ -126,9 +127,22 @@ unsigned long Options::updateSettings()
 
     borderSnapZone = config.readEntry("BorderSnapZone", 10);
     windowSnapZone = config.readEntry("WindowSnapZone", 10);
+    centerSnapZone = config.readEntry("CenterSnapZone", 0);
     snapOnlyWhenOverlapping = config.readEntry("SnapOnlyWhenOverlapping", false);
+
+    // Electric borders
+    KConfigGroup borderConfig( _config, "ElectricBorders" );
+    electric_border_top = electricBorderAction( borderConfig.readEntry( "Top", "None" ));
+    electric_border_top_right = electricBorderAction( borderConfig.readEntry( "TopRight", "None" ));
+    electric_border_right = electricBorderAction( borderConfig.readEntry( "Right", "None" ));
+    electric_border_bottom_right = electricBorderAction( borderConfig.readEntry( "BottomRight", "None" ));
+    electric_border_bottom = electricBorderAction( borderConfig.readEntry( "Bottom", "None" ));
+    electric_border_bottom_left = electricBorderAction( borderConfig.readEntry( "BottomLeft", "None" ));
+    electric_border_left = electricBorderAction( borderConfig.readEntry( "Left", "None" ));
+    electric_border_top_left = electricBorderAction( borderConfig.readEntry( "TopLeft", "None" ));
     electric_borders = config.readEntry("ElectricBorders", 0);
     electric_border_delay = config.readEntry("ElectricBorderDelay", 150);
+    electric_border_cooldown = config.readEntry("ElectricBorderCooldown", 350);
 
     OpTitlebarDblClick = windowOperation( config.readEntry("TitlebarDoubleClickCommand", "Maximize"), true );
     setOpMaxButtonLeftClick( windowOperation( config.readEntry("MaximizeButtonLeftClickCommand", "Maximize"), true ));
@@ -153,7 +167,7 @@ unsigned long Options::updateSettings()
     showDesktopIsMinimizeAll = config.readEntry( "ShowDesktopIsMinimizeAll", false );
 
     // Mouse bindings
-    config = KConfigGroup(_config,"MouseBindings");
+    config = KConfigGroup( _config, "MouseBindings" );
     CmdActiveTitlebar1 = mouseCommand(config.readEntry("CommandActiveTitlebar1","Raise"), true );
     CmdActiveTitlebar2 = mouseCommand(config.readEntry("CommandActiveTitlebar2","Lower"), true );
     CmdActiveTitlebar3 = mouseCommand(config.readEntry("CommandActiveTitlebar3","Operations menu"), true );
@@ -164,6 +178,7 @@ unsigned long Options::updateSettings()
     CmdWindow1 = mouseCommand(config.readEntry("CommandWindow1","Activate, raise and pass click"), false );
     CmdWindow2 = mouseCommand(config.readEntry("CommandWindow2","Activate and pass click"), false );
     CmdWindow3 = mouseCommand(config.readEntry("CommandWindow3","Activate and pass click"), false );
+    CmdWindowWheel = mouseCommand(config.readEntry("CommandWindowWheel", "Scroll"), false);
     CmdAllModKey = (config.readEntry("CommandAllKey","Alt") == "Meta") ? Qt::Key_Meta : Qt::Key_Alt;
     CmdAll1 = mouseCommand(config.readEntry("CommandAll1","Move"), false );
     CmdAll2 = mouseCommand(config.readEntry("CommandAll2","Toggle raise and lower"), false );
@@ -177,15 +192,16 @@ unsigned long Options::updateSettings()
     // Since we want to allow users to enable window decoration tooltips
     // and not kstyle tooltips and vise-versa, we don't read the
     // "EffectNoTooltip" setting from kdeglobals.
-    KConfig _globalConfig( "kdeglobals" );
-    KConfigGroup globalConfig(&_globalConfig, "KDE");
-    topmenus = globalConfig.readEntry( "macStyle", false);
 
-    KConfig _kdesktopcfg( "kdesktoprc" );
-    KConfigGroup kdesktopcfg(&_kdesktopcfg, "Menubar" );
-    desktop_topmenu = kdesktopcfg.readEntry( "ShowMenubar", false);
-    if( desktop_topmenu )
-        topmenus = true;
+#if 0
+    FIXME: we have no mac style menu implementation in kwin anymore, so this just breaks
+           things for people!
+    KConfig _globalConfig("kdeglobals");
+    KConfigGroup globalConfig(&_globalConfig, "KDE");
+    topmenus = globalConfig.readEntry("macStyle", false);
+#else
+    topmenus = false;
+#endif
 
 //    QToolTip::setGloballyEnabled( d->show_tooltips );
 // KDE4 this probably needs to be done manually in clients
@@ -210,6 +226,7 @@ void Options::reloadCompositingSettings(const CompositingPrefs& prefs)
         compositingMode = XRenderCompositing;
     else
         compositingMode = OpenGLCompositing;
+    disableCompositingChecks = config.readEntry("DisableChecks", false);
     QString glmode = config.readEntry("GLMode", "TFP" ).toUpper();
     if( glmode == "TFP" )
         glMode = GLTFP;
@@ -224,11 +241,31 @@ void Options::reloadCompositingSettings(const CompositingPrefs& prefs)
 
     xrenderSmoothScale = config.readEntry("XRenderSmoothScale", false );
 
-    const HiddenPreviews hps[] = { HiddenPreviewsNever, HiddenPreviewsKeep, HiddenPreviewUpdate, HiddenPreviewsActive };
-    hiddenPreviews = hps[ qBound( 0, config.readEntry( "HiddenPreviews", 0 ), 3 ) ];
+    hiddenPreviews = HiddenPreviewsShown;
+    // 4 - off, 5 - shown, 6 - always, other are old values
+    int hps = config.readEntry( "HiddenPreviews", 5 );
+    if( hps == 4 )
+        hiddenPreviews = HiddenPreviewsNever;
+    else if( hps == 5 )
+        hiddenPreviews = HiddenPreviewsShown;
+    else if( hps == 6 )
+        hiddenPreviews = HiddenPreviewsAlways;
+
+    unredirectFullscreen = config.readEntry( "UnredirectFullscreen", true );
+    animationSpeed = qBound( 0, config.readEntry( "AnimationSpeed", 3 ), 6 );
+
+    if( !disableCompositingChecks && !prefs.validateSetup( compositingMode ))
+        useCompositing = false;
     }
 
 
+ElectricBorderAction Options::electricBorderAction( const QString& name )
+    {
+    QString lowerName = name.toLower();
+    if( lowerName == "dashboard" ) return ElectricActionDashboard;
+    return ElectricActionNone;
+    }
+
 // restricted should be true for operations that the user may not be able to repeat
 // if the window is moved out of the workspace (e.g. if the user moves a window
 // by the titlebar, and moves it too high beneath Kicker at the top edge, they
@@ -272,6 +309,9 @@ Options::MouseCommand Options::mouseCommand(const QString &name, bool restricted
     if (lowerName == "activate") return MouseActivate;
     if (lowerName == "activate, raise and pass click") return MouseActivateRaiseAndPassClick;
     if (lowerName == "activate and pass click") return MouseActivateAndPassClick;
+    if (lowerName == "scroll") return MouseNothing;
+    if (lowerName == "activate and scroll") return MouseActivateAndPassClick;
+    if (lowerName == "activate, raise and scroll") return MouseActivateRaiseAndPassClick;
     if (lowerName == "activate, raise and move")
         return restricted ? MouseActivateRaiseAndMove : MouseActivateRaiseAndUnrestrictedMove;
     if (lowerName == "move") return restricted ? MouseMove : MouseUnrestrictedMove;
@@ -299,6 +339,30 @@ bool Options::showGeometryTip()
     return show_geometry_tip;
     }
 
+ElectricBorderAction Options::electricBorderAction( ElectricBorder edge )
+    {
+    switch( edge )
+        {
+        case ElectricTop:
+            return electric_border_top;
+        case ElectricTopRight:
+            return electric_border_top_right;
+        case ElectricRight:
+            return electric_border_right;
+        case ElectricBottomRight:
+            return electric_border_bottom_right;
+        case ElectricBottom:
+            return electric_border_bottom;
+        case ElectricBottomLeft:
+            return electric_border_bottom_left;
+        case ElectricLeft:
+            return electric_border_left;
+        case ElectricTopLeft:
+            return electric_border_top_left;
+        }
+    return ElectricActionNone;
+    }
+
 int Options::electricBorders()
     {
     return electric_borders;
@@ -309,6 +373,11 @@ int Options::electricBorderDelay()
     return electric_border_delay;
     }
 
+int Options::electricBorderCooldown()
+    {
+    return electric_border_cooldown;
+    }
+
 bool Options::checkIgnoreFocusStealing( const Client* c )
     {
     return ignoreFocusStealingClasses.contains(QString::fromLatin1(c->resourceClass()));
@@ -346,4 +415,10 @@ const char* Options::moveResizeModeToString( MoveResizeMode mode )
     return mode == Opaque ? "Opaque" : "Transparent";
     }
 
+double Options::animationTimeFactor() const
+    {
+    const double factors[] = { 0, 0.2, 0.5, 1, 2, 4, 20 };
+    return factors[ animationSpeed ];
+    }
+
 } // namespace
