@@ -79,7 +79,9 @@ class Scene
             // At least one window will be painted with transformed geometry.
             PAINT_SCREEN_WITH_TRANSFORMED_WINDOWS = 1 << 5,
             // Clear whole background as the very first step, without optimizing it
-            PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6
+            PAINT_SCREEN_BACKGROUND_FIRST = 1 << 6,
+            // Temporary solution since (_OPAQUE | _TRANSLUCENT) is not working currently.
+            PAINT_DECORATION_ONLY = 1 << 7
             };
         // types of filtering available
         enum ImageFilterType { ImageFilterFast, ImageFilterGood };
@@ -108,6 +110,10 @@ class Scene
         // compute time since the last repaint
         void updateTimeDiff();
         QList< QPoint > selfCheckPoints() const;
+        QRegion selfCheckRegion() const;
+        // dimensions of the test pixmap for selfcheck
+        int selfCheckWidth() const;
+        int selfCheckHeight() const;
         // saved data for 2nd pass of optimized screen painting
         struct Phase2Data
             {
@@ -133,6 +139,7 @@ class Scene
         QTime last_time;
         Workspace* wspace;
         bool has_waitSync;
+        bool selfCheckDone;
     };
 
 // The base class for windows representations in composite backends
@@ -179,6 +186,7 @@ class Scene::Window
         bool isOpaque() const;
         // shape of the window
         QRegion shape() const;
+        QRegion clientShape() const;
         void discardShape();
         void updateToplevel( Toplevel* c );
         // creates initial quad list for the window
@@ -198,6 +206,18 @@ class Scene::Window
 
 extern Scene* scene;
 
+inline
+int Scene::selfCheckWidth() const
+    {
+    return 3;
+    }
+
+inline
+int Scene::selfCheckHeight() const
+    {
+    return 2;
+    }
+
 inline
 int Scene::Window::x() const
     {
