@@ -4,6 +4,7 @@
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2009 Martin Gräßlin <kde@martin-graesslin.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -25,16 +26,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "workspace.h"
 #include "effects.h"
 #include "client.h"
+#include <QStyleOptionGraphicsItem>
 #include <QPainter>
-#include <QLabel>
-#include <qdrawutil.h>
-#include <QStyle>
+#include <QTimeLine>
 #include <kglobal.h>
 #include <fixx11h.h>
 #include <kconfig.h>
 #include <klocale.h>
-#include <QApplication>
-#include <QDesktopWidget>
 #include <QAction>
 #include <stdarg.h>
 #include <kdebug.h>
@@ -46,6 +44,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kactioncollection.h>
 #include <kkeyserver.h>
 #include <kconfiggroup.h>
+#include <KDE/Plasma/Theme>
+#include <KGlobalSettings>
 
 // specify externals before namespace
 
@@ -55,17 +55,27 @@ namespace KWin
 extern QPixmap* kwin_get_menu_pix_hack();
 
 TabBox::TabBox( Workspace *ws )
-    : QFrame( 0, Qt::X11BypassWindowManagerHint )
+    : QGraphicsView()
     , wspace(ws)
     , client(0)
     , display_refcount( 0 )
+    , selectionItem( 0 )
     {
-    setFrameStyle(QFrame::StyledPanel);
-    setFrameShadow(QFrame::Plain);
-    setBackgroundRole(QPalette::Base);
-    setLineWidth(2);
-    setMidLineWidth(2);
-    setContentsMargins( 2, 2, 2, 2 );
+    scene = new QGraphicsScene( this );
+    setWindowFlags( Qt::X11BypassWindowManagerHint );
+    setScene( scene );
+    setFrameStyle( QFrame::NoFrame );
+    viewport()->setAutoFillBackground( false );
+    setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
+    setAttribute( Qt::WA_TranslucentBackground );
+    frame.setImagePath( "dialogs/background" );
+    frame.setCacheAllRenderedFrames( true );
+    frame.setEnabledBorders( Plasma::FrameSvg::AllBorders );
+    item_frame.setImagePath( "widgets/viewitem" );
+    item_frame.setElementPrefix( "hover" );
+    item_frame.setCacheAllRenderedFrames( true );
+    item_frame.setEnabledBorders( Plasma::FrameSvg::AllBorders );
 
     showMiniIcon = false;
 
@@ -74,10 +84,24 @@ TabBox::TabBox( Workspace *ws )
     reconfigure();
     reset();
     connect(&delayedShowTimer, SIGNAL(timeout()), this, SLOT(show()));
+    XSetWindowAttributes attr;
+    attr.override_redirect = 1;
+    outline_left = XCreateWindow( QX11Info::display(), QX11Info::appRootWindow(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_right = XCreateWindow( QX11Info::display(), QX11Info::appRootWindow(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_top = XCreateWindow( QX11Info::display(), QX11Info::appRootWindow(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
+    outline_bottom = XCreateWindow( QX11Info::display(), QX11Info::appRootWindow(), 0, 0, 1, 1, 0,
+        CopyFromParent, CopyFromParent, CopyFromParent, CWOverrideRedirect, &attr );
     }
 
 TabBox::~TabBox()
     {
+    XDestroyWindow( QX11Info::display(), outline_left );
+    XDestroyWindow( QX11Info::display(), outline_right );
+    XDestroyWindow( QX11Info::display(), outline_top );
+    XDestroyWindow( QX11Info::display(), outline_bottom );
     }
 
 
@@ -188,12 +212,14 @@ void TabBox::createDesktopList(QList< int > &list, int start, SortOrder order)
 void TabBox::reset( bool partial_reset )
     {
     int w, h, cw = 0, wmax = 0;
+    qreal left, top, right, bottom;
+    frame.getMargins( left, top, right, bottom );
 
     QRect r = workspace()->screenGeometry( workspace()->activeScreen());
 
     // calculate height of 1 line
-    // fontheight + 1 pixel above + 1 pixel below, or 32x32 icon + 2 pixel above + below
-    lineHeight = qMax(fontMetrics().height() + 2, 32 + 4);
+    // fontheight + 2 pixel above + 2 pixel below, or 32x32 icon + 4 pixel above + below
+    lineHeight = top + bottom + qMax(QFontMetrics( KGlobalSettings::smallestReadableFont() ).height(), 32 );
 
     if ( mode() == TabBoxWindowsMode )
         {
@@ -207,10 +233,15 @@ void TabBox::reset( bool partial_reset )
         createClientList(clients, options_traverse_all ? -1 : workspace()->currentDesktop(), starting_client, true);
 
         // calculate maximum caption width
-        cw = fontMetrics().width(no_tasks)+20;
-        for (ClientList::ConstIterator it = clients.begin(); it != clients.end(); ++it)
+        cw = fontMetrics().width(no_tasks) + 20;
+        QFont f = font();
+        f.setPointSize( KGlobalSettings::smallestReadableFont().pointSize() );
+        QFontMetrics fm = QFontMetrics( f );
+        for (ClientList::ConstIterator it = clients.constBegin(); it != clients.constEnd(); ++it)
           {
-          cw = fontMetrics().width( (*it)->caption() );
+          cw = fm.width( (*it)->caption() );
+          if( (*it)->desktop() != wspace->currentDesktop() )
+            cw += fm.width( QString( wspace->desktopName((*it)->desktop()) + ": " ) );
           if ( cw > wmax ) wmax = cw;
           }
 
@@ -228,18 +259,18 @@ void TabBox::reset( bool partial_reset )
           showMiniIcon = false;
           h = clients.count() * lineHeight;
 
-          if ( h > (r.height()-(2*frameWidth())) )  // if too high, use mini icons
+          if ( h > (r.height()-(top + bottom)) )  // if too high, use mini icons
             {
             showMiniIcon = true;
-            // fontheight + 1 pixel above + 1 pixel below, or 16x16 icon + 1 pixel above + below
-            lineHeight = qMax(fontMetrics().height() + 2, 16 + 2);
+            // fontheight + 1 pixel above + 1 pixel below, or 16x16 icon + 2 pixel above + below
+            lineHeight = qMax(QFontMetrics( KGlobalSettings::smallestReadableFont() ).height() + 2, 16 + 4);
 
             h = clients.count() * lineHeight;
 
-            if ( h > (r.height()-(2*frameWidth())) ) // if still too high, remove some clients
+            if ( h > (r.height()-(top + bottom)) ) // if still too high, remove some clients
               {
                 // how many clients to remove
-                int howMany = (h - (r.height()-(2*frameWidth())))/lineHeight;
+                int howMany = (h - (r.height()-(top + bottom)))/lineHeight + 1;
                 for (; howMany; howMany--)
                   clients.removeAll(clients.last());
 
@@ -278,18 +309,103 @@ void TabBox::reset( bool partial_reset )
         }
 
     // height, width for the popup
-    h += 2 * frameWidth();
-    w = 2*frameWidth() + 5*2 + ( showMiniIcon ? 16 : 32 ) + 8 + wmax; // 5*2=margins, ()=icon, 8=space between icon+text
-    w = qBound( r.width()/3 , w, r.width() * 4 / 5 );
+    h += top + bottom;
+    h = qMin( h, r.height() );
+    w = left + right + 5*2 + ( showMiniIcon ? 16 : 32 ) + 8 + wmax; // 5*2=margins, ()=icon, 8=space between icon+text
+    w = qBound( r.width()/5 , w, r.width() * 4 / 5 );
 
-    setGeometry( (r.width()-w)/2 + r.x(),
+    setGeometry((r.width()-w)/2 + r.x(),
                  (r.height()-h)/2+ r.y(),
                  w, h );
+    scene->setSceneRect( 0, 0, w, h );
+    frame.resizeFrame( QSize( w , h ) );
+    // resizing the item frame
+    item_frame.resizeFrame( QSize( w-left-right, lineHeight ) );
+
+    setMask( frame.mask() );
+
+    if( partial_reset )
+        initScene();
 
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->tabBoxUpdated();
     }
 
+void TabBox::initScene()
+    {
+    scene->clear();
+    qreal left, top, right, bottom;
+    frame.getMargins( left, top, right, bottom );
+
+    if( KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects )
+        {
+        selectionItem = new TabBoxSelectionItem( this );
+        selectionItem->setPos( left, top );
+        selectionItem->hide();
+        scene->addItem( selectionItem );
+        }
+    else
+        {
+        selectionItem = 0;
+        }
+
+    if( mode() == TabBoxWindowsMode )
+        {
+        if( clients.isEmpty() )
+            {
+            QFont f = font();
+            f.setBold( true );
+            f.setPointSize( 14 );
+
+            QGraphicsTextItem* item = scene->addText( no_tasks, f );
+            item->setDefaultTextColor( Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor ) );
+            item->adjustSize();
+            item->setPos( width()*0.5 - QFontMetrics(f).width( item->toPlainText() )*0.5,
+                height()*0.5 - QFontMetrics(f).height()*0.5 );
+            }
+        else
+            {
+            // add clients to scene
+            int index = 0;
+
+            foreach( Client* c, clients )
+                {
+                TabBoxWindowItem* item = new TabBoxWindowItem( c, this );
+                item->setHeight( lineHeight );
+                item->setWidth( width() - left - right );
+                item->setShowMiniIcons( showMiniIcon );
+                item->setPos( left, top + lineHeight * index );
+                scene->addItem( item );
+
+                if ( selectionItem && c == client )
+                    {
+                    selectionItem->setPos( item->pos() );
+                    }
+
+                index++;
+                }
+
+            }
+        }
+    else
+        { // TabBoxDesktopMode || TabBoxDesktopListMode
+        int y = top;
+
+        QRect r = workspace()->screenGeometry( workspace()->activeScreen());
+        foreach (int it, desktops)
+            {
+            TabBoxDesktopItem* item = new TabBoxDesktopItem( it, this );
+            item->setHeight( lineHeight );
+            item->setWidth( width() - left - right );
+            item->setPos( left, top + lineHeight * (it-1) );
+            scene->addItem( item );
+            // next desktop
+            y += lineHeight;
+            if ( y >= r.height() )
+                break;
+            }
+        }
+    }
 
 /*!
   Shows the next or previous item, depending on \a next
@@ -334,7 +450,6 @@ void TabBox::nextPrev( bool next)
 
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->tabBoxUpdated();
-    update();
     }
 
 
@@ -398,9 +513,32 @@ QList< int > TabBox::currentDesktopList()
  */
 void TabBox::setCurrentClient( Client* newClient )
     {
+    foreach( QGraphicsItem *item, scene->items() )
+        {
+        TabBoxWindowItem* wItem = dynamic_cast<TabBoxWindowItem*>( item );
+        if( !wItem )
+            continue;
+
+        if ( wItem->client() == newClient )
+            {
+            if (selectionItem)
+                selectionItem->moveTo( wItem );
+            else
+                wItem->update();
+            }
+        else if ( !selectionItem && wItem->client() == client )
+            {
+                wItem->update();
+            }
+        }
+
     client = newClient;
+
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->tabBoxUpdated();
+
+    if( isVisible() )
+        updateOutline();
     }
 
 /*!
@@ -410,7 +548,27 @@ void TabBox::setCurrentClient( Client* newClient )
  */
 void TabBox::setCurrentDesktop( int newDesktop )
     {
+    foreach( QGraphicsItem *item, scene->items() )
+        {
+        TabBoxDesktopItem* dItem = dynamic_cast<TabBoxDesktopItem*>( item );
+        if( !dItem )
+            continue;
+
+        if ( dItem->desktop() == newDesktop )
+            {
+            if (selectionItem)
+                selectionItem->moveTo( dItem );
+            else
+                dItem->update();
+            }
+        else if ( !selectionItem && dItem->desktop() == desk )
+            {
+                dItem->update();
+            }
+        }
+
     desk = newDesktop;
+
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->tabBoxUpdated();
     }
@@ -420,6 +578,8 @@ void TabBox::setCurrentDesktop( int newDesktop )
  */
 void TabBox::showEvent( QShowEvent* )
     {
+    if( isVisible() )
+        updateOutline();
     raise();
     }
 
@@ -431,181 +591,14 @@ void TabBox::hideEvent( QHideEvent* )
     {
     }
 
-/*!
-  Paints the tab box
- */
-void TabBox::paintEvent( QPaintEvent* e )
+void TabBox::drawBackground( QPainter* painter, const QRectF& rect )
     {
-    QFrame::paintEvent( e );
-
-    QPainter p( this );
-    QRect r( contentsRect());
-
-    QPixmap* menu_pix = kwin_get_menu_pix_hack();
-
-    int iconWidth = showMiniIcon ? 16 : 32;
-    int x = r.x();
-    int y = r.y();
-
-    if ( mode () == TabBoxWindowsMode )
-        {
-        if ( !currentClient() )
-            {
-            QFont f = font();
-            f.setBold( true );
-            f.setPointSize( 14 );
-
-            p.setFont(f);
-            p.drawText( r, Qt::AlignCenter, no_tasks);
-            }
-        else
-            {
-            for (ClientList::ConstIterator it = clients.begin(); it != clients.end(); ++it)
-              {
-              if ( workspace()->hasClient( *it ) )  // safety
-                  {
-                  // draw highlight background
-                  if ( (*it) == currentClient() )
-                    p.fillRect(x, y, r.width(), lineHeight, palette().brush( QPalette::Active, QPalette::Highlight ));
-
-                  // draw icon
-                  QPixmap icon;
-                  if ( showMiniIcon )
-                    {
-                    if ( !(*it)->miniIcon().isNull() )
-                      icon = (*it)->miniIcon();
-                    }
-                  else
-                    if ( !(*it)->icon().isNull() )
-                      icon = (*it)->icon();
-                    else if ( menu_pix )
-                      icon = *menu_pix;
-                
-                  if( !icon.isNull())
-                    {
-                    if( (*it)->isMinimized())
-                        KIconEffect::semiTransparent( icon );
-                    p.drawPixmap( x+5, y + (lineHeight - iconWidth)/2, icon );
-                    }
-
-                  // generate text to display
-                  QString s;
-
-                  if ( !(*it)->isOnDesktop(workspace()->currentDesktop()) )
-                    s = workspace()->desktopName((*it)->desktop()) + ": ";
-
-                  if ( (*it)->isMinimized() )
-                    s += '(' + (*it)->caption() + ')';
-                  else
-                    s += (*it)->caption();
-
-                  s = fontMetrics().elidedText( s, Qt::ElideMiddle, r.width() - 5 - iconWidth - 8 );
-
-                  // draw text
-                  if ( (*it) == currentClient() )
-                    p.setPen(palette().color( QPalette::Active, QPalette::HighlightedText ));
-                  else if( (*it)->isMinimized())
-                    {
-                    QColor c1 = palette().color( QPalette::Active, QPalette::Text );
-                    QColor c2 = palette().color( QPalette::Active, QPalette::Background );
-                    // from kicker's TaskContainer::blendColors()
-                    int r1, g1, b1;
-                    int r2, g2, b2;
-
-                    c1.getRgb( &r1, &g1, &b1 );
-                    c2.getRgb( &r2, &g2, &b2 );
-
-                    r1 += (int) ( .5 * ( r2 - r1 ) );
-                    g1 += (int) ( .5 * ( g2 - g1 ) );
-                    b1 += (int) ( .5 * ( b2 - b1 ) );
-
-                    p.setPen(QColor( r1, g1, b1 ));
-                    }
-                  else
-                    p.setPen(palette().color( QPalette::Active, QPalette::Text ));
-
-                  p.drawText(x+5 + iconWidth + 8, y, r.width() - 5 - iconWidth - 8, lineHeight,
-                              Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine, s);
-
-                  y += lineHeight;
-                  }
-              if ( y >= r.height() ) break;
-              }
-            }
-        }
-    else
-        { // TabBoxDesktopMode || TabBoxDesktopListMode
-        int iconHeight = iconWidth;
-
-        // get widest desktop name/number
-        QFont f(font());
-        f.setBold(true);
-        f.setPixelSize(iconHeight - 4);  // pixel, not point because I need to know the pixels
-        QFontMetrics fm(f);
-
-        int wmax = 0;
-        foreach (int it, desktops)
-            {
-            wmax = qMax(wmax, fontMetrics().width(workspace()->desktopName(it)));
-
-            // calculate max width of desktop-number text
-            QString num = QString::number(it);
-            iconWidth = qMax(iconWidth - 4, fm.boundingRect(num).width()) + 4;
-            }
-
-        foreach (int it, desktops)
-            {
-            // draw highlight background
-            if ( it == desk )  // current desktop
-              p.fillRect(x, y, r.width(), lineHeight, palette().brush( QPalette::Active, QPalette::Highlight ));
-
-            p.save();
-
-            // draw "icon" (here: number of desktop)
-            p.fillRect(x+5, y+2, iconWidth, iconHeight, palette().brush( QPalette::Active, QPalette::Base ));
-            p.setPen(palette().color( QPalette::Active, QPalette::Text ));
-            p.drawRect(x+5, y+2, iconWidth, iconHeight);
-
-            // draw desktop-number
-            p.setFont(f);
-            QString num = QString::number(it);
-            p.drawText(x+5, y+2, iconWidth, iconHeight, Qt::AlignCenter, num);
-
-            p.restore();
-
-            // draw desktop name text
-            if ( it == desk )
-              p.setPen(palette().color( QPalette::Active, QPalette::HighlightedText ));
-            else
-              p.setPen(palette().color( QPalette::Active, QPalette::Text ));
-
-            p.drawText(x+5 + iconWidth + 8, y, r.width() - 5 - iconWidth - 8, lineHeight,
-                       Qt::AlignLeft | Qt::AlignVCenter | Qt::TextSingleLine,
-                       workspace()->desktopName(it));
-
-            // show mini icons from that desktop aligned to each other
-            int x1 = x + 5 + iconWidth + 8 + wmax + 5;
-
-            ClientList list;
-            createClientList(list, it, 0, false);
-            // clients are in reversed stacking order
-            for ( int i = list.size() - 1; i>=0; i-- )
-              {
-              if ( !list.at( i )->miniIcon().isNull() )
-                {
-                if ( x1+18 >= x+r.width() )  // only show full icons
-                  break;
-
-                p.drawPixmap( x1, y + (lineHeight - 16)/2, list.at(  i )->miniIcon() );
-                x1 += 18;
-                }
-              }
-
-            // next desktop
-            y += lineHeight;
-            if ( y >= r.height() ) break;
-            }
-        }
+    painter->save();
+    painter->setCompositionMode( QPainter::CompositionMode_Source );
+    qreal left, top, right, bottom;
+    frame.getMargins( left, top, right, bottom );
+    frame.paintFrame( painter, rect.adjusted( -left, -top, right, bottom ) );
+    painter->restore();
     }
 
 
@@ -623,6 +616,90 @@ void TabBox::show()
     QWidget::show();
     }
 
+void TabBox::updateOutline()
+    {
+    Client* c = currentClient();
+    if( c == NULL || this->isHidden() || !c->isShown( true ) || !c->isOnCurrentDesktop())
+        {
+        XUnmapWindow( QX11Info::display(), outline_left );
+        XUnmapWindow( QX11Info::display(), outline_right );
+        XUnmapWindow( QX11Info::display(), outline_top );
+        XUnmapWindow( QX11Info::display(), outline_bottom );
+        return;
+        }
+    // left/right parts are between top/bottom, they don't reach as far as the corners
+    XMoveResizeWindow( QX11Info::display(), outline_left, c->x(), c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( QX11Info::display(), outline_right, c->x() + c->width() - 5, c->y() + 5, 5, c->height() - 10 );
+    XMoveResizeWindow( QX11Info::display(), outline_top, c->x(), c->y(), c->width(), 5 );
+    XMoveResizeWindow( QX11Info::display(), outline_bottom, c->x(), c->y() + c->height() - 5, c->width(), 5 );
+    {
+    QPixmap pix( 5, c->height() - 10 );
+    QPainter p( &pix );
+    p.setPen( Qt::white );
+    p.drawLine( 0, 0, 0, pix.height() - 1 );
+    p.drawLine( 4, 0, 4, pix.height() - 1 );
+    p.setPen( Qt::gray );
+    p.drawLine( 1, 0, 1, pix.height() - 1 );
+    p.drawLine( 3, 0, 3, pix.height() - 1 );
+    p.setPen( Qt::black );
+    p.drawLine( 2, 0, 2, pix.height() - 1 );
+    p.end();
+    XSetWindowBackgroundPixmap( QX11Info::display(), outline_left, pix.handle());
+    XSetWindowBackgroundPixmap( QX11Info::display(), outline_right, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( Qt::white );
+    p.drawLine( 0, 0, pix.width() - 1 - 0, 0 );
+    p.drawLine( 4, 4, pix.width() - 1 - 4, 4 );
+    p.drawLine( 0, 0, 0, 4 );
+    p.drawLine( pix.width() - 1 - 0, 0, pix.width() - 1 - 0, 4 );
+    p.setPen( Qt::gray );
+    p.drawLine( 1, 1, pix.width() - 1 - 1, 1 );
+    p.drawLine( 3, 3, pix.width() - 1 - 3, 3 );
+    p.drawLine( 1, 1, 1, 4 );
+    p.drawLine( 3, 3, 3, 4 );
+    p.drawLine( pix.width() - 1 - 1, 1, pix.width() - 1 - 1, 4 );
+    p.drawLine( pix.width() - 1 - 3, 3, pix.width() - 1 - 3, 4 );
+    p.setPen( Qt::black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 2, 2, 4 );
+    p.drawLine( pix.width() - 1 - 2, 2, pix.width() - 1 - 2, 4 );
+    p.end();
+    XSetWindowBackgroundPixmap( QX11Info::display(), outline_top, pix.handle());
+    }
+    {
+    QPixmap pix( c->width(), 5 );
+    QPainter p( &pix );
+    p.setPen( Qt::white );
+    p.drawLine( 4, 0, pix.width() - 1 - 4, 0 );
+    p.drawLine( 0, 4, pix.width() - 1 - 0, 4 );
+    p.drawLine( 0, 4, 0, 0 );
+    p.drawLine( pix.width() - 1 - 0, 4, pix.width() - 1 - 0, 0 );
+    p.setPen( Qt::gray );
+    p.drawLine( 3, 1, pix.width() - 1 - 3, 1 );
+    p.drawLine( 1, 3, pix.width() - 1 - 1, 3 );
+    p.drawLine( 3, 1, 3, 0 );
+    p.drawLine( 1, 3, 1, 0 );
+    p.drawLine( pix.width() - 1 - 3, 1, pix.width() - 1 - 3, 0 );
+    p.drawLine( pix.width() - 1 - 1, 3, pix.width() - 1 - 1, 0 );
+    p.setPen( Qt::black );
+    p.drawLine( 2, 2, pix.width() - 1 - 2, 2 );
+    p.drawLine( 2, 0, 2, 2 );
+    p.drawLine( pix.width() - 1 - 2, 0, pix.width() - 1 - 2, 2 );
+    p.end();
+    XSetWindowBackgroundPixmap( QX11Info::display(), outline_bottom, pix.handle());
+    }
+    XClearWindow( QX11Info::display(), outline_left );
+    XClearWindow( QX11Info::display(), outline_right );
+    XClearWindow( QX11Info::display(), outline_top );
+    XClearWindow( QX11Info::display(), outline_bottom );
+    XMapWindow( QX11Info::display(), outline_left );
+    XMapWindow( QX11Info::display(), outline_right );
+    XMapWindow( QX11Info::display(), outline_top );
+    XMapWindow( QX11Info::display(), outline_bottom );
+    }
 
 /*!
   Notify effects that the tab box is being hidden.
@@ -631,7 +708,13 @@ void TabBox::hide()
     {
     delayedShowTimer.stop();
     if( isVisible())
+        {
         unrefDisplay();
+        XUnmapWindow( QX11Info::display(), outline_left );
+        XUnmapWindow( QX11Info::display(), outline_right );
+        XUnmapWindow( QX11Info::display(), outline_top );
+        XUnmapWindow( QX11Info::display(), outline_bottom );
+        }
     if( effects )
         static_cast<EffectsHandlerImpl*>(effects)->tabBoxClosed();
     if( isDisplayed())
@@ -715,12 +798,14 @@ void TabBox::handleMouseEvent( XEvent* e )
         return;
         }
 
-    int num = (widgetPos.y()-frameWidth()) / lineHeight;
+    qreal left, top, right, bottom;
+    frame.getMargins( left, top, right, bottom );
+    int num = (widgetPos.y()-top) / lineHeight;
 
     if( mode() == TabBoxWindowsMode )
         {
-        for( ClientList::ConstIterator it = clients.begin();
-             it != clients.end();
+        for( ClientList::ConstIterator it = clients.constBegin();
+             it != clients.constEnd();
              ++it)
             {
             if( workspace()->hasClient( *it ) && (num == 0) ) // safety
@@ -743,9 +828,312 @@ void TabBox::handleMouseEvent( XEvent* e )
             num--;
             }
         }
-    update();
     }
 
+
+//*******************************
+// TabBoxSelectionItem
+//*******************************
+TabBoxSelectionItem::TabBoxSelectionItem( TabBox* parent )
+    : QObject()
+    , QGraphicsItem()
+    , m_parent( parent )
+    , m_timeLine( new QTimeLine( 100, this ) )
+    {
+    setCacheMode( DeviceCoordinateCache );
+    setZValue( -1000 );
+    setFlag(ItemIsMovable, false);
+    setFlag(ItemIsSelectable, false);
+    setFlag(ItemIsFocusable, false);
+
+    m_timeLine->setFrameRange( 0, 40 );
+    m_timeLine->setCurveShape( QTimeLine::EaseInCurve );
+    connect( m_timeLine, SIGNAL( valueChanged(qreal) ),
+             this, SLOT( animateMove(qreal) ) );
+    }
+
+TabBoxSelectionItem::~TabBoxSelectionItem()
+    {
+    }
+
+QRectF TabBoxSelectionItem::boundingRect() const
+    {
+    return m_boundingRect;
+    }
+
+void TabBoxSelectionItem::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
+    {
+    Q_UNUSED( widget );
+    painter->setRenderHint( QPainter::Antialiasing );
+    m_parent->itemFrame()->paintFrame( painter, option->exposedRect, option->exposedRect );
+    }
+
+void TabBoxSelectionItem::moveTo( QGraphicsItem * item )
+    {
+    if (m_boundingRect != item->boundingRect())
+        {
+        prepareGeometryChange();
+        m_boundingRect = item->boundingRect();
+        }
+
+    show();
+
+    if (!m_animEndRect.isEmpty()) {
+        setPos(m_animEndRect.topLeft());
+    }
+
+    m_animStartRect = QRectF( pos(), boundingRect().size() );
+    m_animEndRect = QRectF( item->pos(), item->boundingRect().size() );
+    m_timeLine->stop();
+    m_timeLine->start();
+    }
+
+void TabBoxSelectionItem::animateMove( qreal t )
+    {
+        setPos(m_animStartRect.topLeft() * (1 - t) + m_animEndRect.topLeft() * t);
+        if (qFuzzyCompare(t, qreal(1.0))) {
+            m_animEndRect = QRect();
+        }
+    }
+
+//*******************************
+// TabBoxWindowItem
+//*******************************
+
+TabBoxWindowItem::TabBoxWindowItem( Client* client, TabBox* parent )
+    : QGraphicsItem()
+    , m_client( client )
+    , m_parent( parent )
+    , m_width( 0 )
+    , m_height( 0 )
+    , m_showMiniIcons( false )
+    {
+    setCacheMode( DeviceCoordinateCache );
+    }
+
+TabBoxWindowItem::~TabBoxWindowItem()
+    {
+    }
+
+QRectF TabBoxWindowItem::boundingRect() const
+    {
+    return QRectF( 0, 0, m_width, m_height );
+    }
+
+void TabBoxWindowItem::setHeight( int height )
+    {
+    m_height = height;
+    }
+
+void TabBoxWindowItem::setWidth( int width )
+    {
+    m_width = width;
+    }
+
+void TabBoxWindowItem::setShowMiniIcons( bool showMiniIcons )
+    {
+    m_showMiniIcons = showMiniIcons;
+    }
+
+Client* TabBoxWindowItem::client() const
+{
+    return m_client;
+}
+
+void TabBoxWindowItem::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
+    {
+    painter->setRenderHint( QPainter::Antialiasing );
+
+    drawBackground( painter, option, widget );
+
+    // draw icon
+    int iconWidth = m_showMiniIcons ? 16 : 32;
+    int x = 0;
+    QPixmap* menu_pix = kwin_get_menu_pix_hack();
+    QPixmap icon;
+    if ( m_showMiniIcons )
+        {
+        if ( !m_client->miniIcon().isNull() )
+            icon = m_client->miniIcon();
+        }
+    else
+        {
+        if ( !m_client->icon().isNull() )
+            icon = m_client->icon();
+        else if ( menu_pix )
+            icon = *menu_pix;
+        }
+
+    const int iconX = x + 5;
+    const int iconY = ( m_height - iconWidth ) / 2;
+    if( !icon.isNull() )
+        {
+        if( m_client->isMinimized())
+            KIconEffect::semiTransparent( icon );
+        if( m_client == m_parent->currentClient() )
+            {
+            KIconEffect *effect = KIconLoader::global()->iconEffect();
+            icon = effect->apply( icon, KIconLoader::Desktop, KIconLoader::ActiveState );
+            }
+        painter->drawPixmap( iconX, iconY, icon );
+        }
+
+    // generate text to display
+    QString s;
+
+    if ( !m_client->isOnDesktop(m_parent->workspace()->currentDesktop()) )
+        s = m_parent->workspace()->desktopName(m_client->desktop()).append(": ");
+
+    int textOptions = Qt::AlignLeft | Qt::AlignBottom | Qt ::TextSingleLine;
+    QFont font = painter->font();
+    font.setPointSize( KGlobalSettings::smallestReadableFont().pointSize() );
+
+    if ( m_client->isMinimized() )
+        {
+        s += '(' + m_client->caption() + ')';
+        font.setItalic( true );
+        }
+    else
+        s += m_client->caption();
+
+    QFontMetrics fm = QFontMetrics( font );
+    const int textWidth = m_width - iconX - iconWidth;
+    int textHeight = m_height - (iconX * 2);
+
+    if (!m_showMiniIcons && fm.height() < m_height * 2 / 3) {
+        textHeight = m_height - (m_height / 3);
+    }
+
+    s = fm.elidedText( s, Qt::ElideMiddle, textWidth );
+    painter->setPen( Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor ) );
+    painter->setFont( font );
+    painter->drawText( iconX + iconWidth + 8, iconY, textWidth, textHeight, textOptions, s );
+    }
+
+void TabBoxWindowItem::drawBackground( QPainter* painter, const QStyleOptionGraphicsItem* , QWidget* )
+    {
+    if( m_parent->itemsDrawBackgrounds() && m_client == m_parent->currentClient() )
+        {
+        m_parent->itemFrame()->paintFrame( painter, boundingRect() );
+        }
+    }
+
+
+//*******************************
+// TabBoxDesktopItem
+//*******************************
+
+TabBoxDesktopItem::TabBoxDesktopItem( int desktop, TabBox* parent )
+    : QGraphicsItem()
+    , m_desktop( desktop )
+    , m_parent( parent )
+    , m_width( 0 )
+    , m_height( 0 )
+    {
+    setCacheMode( DeviceCoordinateCache );
+    }
+
+TabBoxDesktopItem::~TabBoxDesktopItem()
+    {
+    }
+
+QRectF TabBoxDesktopItem::boundingRect() const
+    {
+    return QRectF( 0, 0, m_width, m_height );
+    }
+
+void TabBoxDesktopItem::setHeight( int height )
+    {
+    m_height = height;
+    }
+
+void TabBoxDesktopItem::setWidth( int width )
+    {
+    m_width = width;
+    }
+
+int TabBoxDesktopItem::desktop() const
+    {
+    return m_desktop;
+    }
+
+void TabBoxDesktopItem::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
+    {
+    painter->setRenderHint( QPainter::Antialiasing );
+
+    drawBackground( painter, option, widget );
+
+    int iconWidth = 32;
+    int iconHeight = iconWidth;
+    int x = 0;
+    int y = 0;
+
+    // get widest desktop name/number
+    QFont f = painter->font();
+    f.setBold(true);
+    f.setPixelSize(iconHeight - 4);  // pixel, not point because I need to know the pixels
+    QFontMetrics fm = QFontMetrics( f );
+
+    QString num = QString::number( m_desktop );
+    iconWidth = qMax(iconWidth - 4, fm.boundingRect(num).width()) + 4;
+
+    // draw "icon" (here: number of desktop)
+    painter->save();
+    QBrush brush;
+    if( m_desktop == m_parent->currentDesktop() )
+        brush = m_parent->palette().brush( QPalette::Active, QPalette::Highlight );
+    else
+        brush = m_parent->palette().brush( QPalette::Active, QPalette::Base );
+    painter->fillRect(x+5, y+4, iconWidth, iconHeight, brush );
+    painter->setPen(m_parent->palette().color( QPalette::Active, QPalette::Text ));
+    painter->drawRect(x+5, y+4, iconWidth, iconHeight);
+
+    // draw desktop-number
+    painter->setFont(f);
+    painter->drawText(x+5, y+4, iconWidth, iconHeight, Qt::AlignCenter | Qt::AlignVCenter, num);
+
+    painter->restore();
+
+    // draw desktop name text
+    QFont font = painter->font();
+    font.setPointSize( KGlobalSettings::smallestReadableFont().pointSize() );
+    painter->setFont( font );
+    fm = QFontMetrics( font );
+    int wmax = fm.width( m_parent->workspace()->desktopName( m_desktop ));
+
+    painter->setPen( Plasma::Theme::defaultTheme()->color( Plasma::Theme::TextColor ) );
+    painter->drawText(x+5 + iconWidth + 8, y, m_width - 5 - iconWidth - 8, m_height - 4,
+                    Qt::AlignLeft | Qt::AlignBottom | Qt::TextSingleLine,
+                    m_parent->workspace()->desktopName( m_desktop ));
+
+    // show mini icons from that desktop aligned to each other
+    int x1 = x + 5 + iconWidth + 8 + wmax + 5;
+
+    ClientList list;
+    m_parent->createClientList(list, m_desktop, 0, false);
+    // clients are in reversed stacking order
+    for ( int i = list.size() - 1; i>=0; i-- )
+        {
+        if ( !list.at( i )->miniIcon().isNull() )
+            {
+            if ( x1+18 >= m_width )  // only show full icons
+                break;
+
+            painter->drawPixmap( x1, y + (m_height - 16)/2, list.at(  i )->miniIcon() );
+            x1 += 18;
+            }
+        }
+    }
+
+void TabBoxDesktopItem::drawBackground( QPainter* painter, const QStyleOptionGraphicsItem*, QWidget* )
+    {
+    if( m_parent->itemsDrawBackgrounds() && m_desktop == m_parent->currentDesktop() )
+        {
+        m_parent->itemFrame()->paintFrame( painter, boundingRect() );
+        }
+    }
+
+
 //*******************************
 // Workspace
 //*******************************
@@ -950,6 +1338,36 @@ void Workspace::slotWalkBackThroughDesktopList()
         }
     }
 
+void Workspace::slotWalkThroughDesktopsKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughDesktops = KShortcut( seq );
+    }
+
+void Workspace::slotWalkBackThroughDesktopsKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughDesktopsReverse = KShortcut( seq );
+    }
+
+void Workspace::slotWalkThroughDesktopListKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughDesktopList = KShortcut( seq );
+    }
+
+void Workspace::slotWalkBackThroughDesktopListKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughDesktopListReverse = KShortcut( seq );
+    }
+
+void Workspace::slotWalkThroughWindowsKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughWindows = KShortcut( seq );
+    }
+
+void Workspace::slotWalkBackThroughWindowsKeyChanged( const QKeySequence& seq )
+    {
+    cutWalkThroughWindowsReverse = KShortcut( seq );
+    }
+
 void Workspace::modalActionsSwitch( bool enabled )
     {
     QList<KActionCollection*> collections;
@@ -969,6 +1387,7 @@ bool Workspace::startKDEWalkThroughWindows()
     modalActionsSwitch( false );
     tab_box->setMode( TabBoxWindowsMode );
     tab_box->reset();
+    tab_box->initScene();
     return true;
     }
 
@@ -980,6 +1399,7 @@ bool Workspace::startWalkThroughDesktops( TabBoxMode mode )
     modalActionsSwitch( false );
     tab_box->setMode( mode );
     tab_box->reset();
+    tab_box->initScene();
     return true;
     }
 
@@ -1264,7 +1684,7 @@ int Workspace::previousDesktopStatic( int iDesktop ) const
     }
 
 /*!
-  auxiliary functions to travers all clients according the focus
+  auxiliary functions to travers all clients according to the focus
   order. Useful for kwms Alt-tab feature.
 */
 Client* Workspace::nextClientFocusChain( Client* c ) const
@@ -1281,7 +1701,7 @@ Client* Workspace::nextClientFocusChain( Client* c ) const
     }
 
 /*!
-  auxiliary functions to travers all clients according the focus
+  auxiliary functions to travers all clients according to the focus
   order. Useful for kwms Alt-tab feature.
 */
 Client* Workspace::previousClientFocusChain( Client* c ) const
@@ -1298,7 +1718,7 @@ Client* Workspace::previousClientFocusChain( Client* c ) const
     }
 
 /*!
-  auxiliary functions to travers all clients according the static
+  auxiliary functions to travers all clients according to the static
   order. Useful for the CDE-style Alt-tab feature.
 */
 Client* Workspace::nextClientStatic( Client* c ) const
@@ -1314,7 +1734,7 @@ Client* Workspace::nextClientStatic( Client* c ) const
     return clients[ pos ];
     }
 /*!
-  auxiliary functions to travers all clients according the static
+  auxiliary functions to travers all clients according to the static
   order. Useful for the CDE-style Alt-tab feature.
 */
 Client* Workspace::previousClientStatic( Client* c ) const
