@@ -40,11 +40,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <kactioncollection.h>
 #include <kaction.h>
 #include <kconfiggroup.h>
+#include <kcmdlineargs.h>
 #include <QtDBus/QtDBus>
 
 #include "client.h"
-#include "popupinfo.h"
 #include "tabbox.h"
+#include "desktopchangeosd.h"
 #include "atoms.h"
 #include "placement.h"
 #include "notifications.h"
@@ -87,8 +88,13 @@ Workspace* Workspace::_self = 0;
 
 Workspace::Workspace( bool restore )
     : QObject( 0 )
-    , current_desktop( 0 )
-    , number_of_desktops( 0 )
+    // Desktop layout
+    , desktopCount_( 0 ) // This is an invalid state
+    , desktopGridSize_( 1, 2 ) // Default to two rows
+    , desktopGrid_( new int[2] )
+    , currentDesktop_( 0 )
+    , desktopLayoutDynamicity_( false )
+    // Unsorted
     , active_popup( NULL )
     , active_popup_client( NULL )
     , temporaryRulesMessages( "_KDE_NET_WM_TEMPORARY_RULES", NULL, false )
@@ -111,7 +117,7 @@ Workspace::Workspace( bool restore )
     , mouse_emulation( false )
     , block_focus( 0 )
     , tab_box( 0 )
-    , popupinfo( 0 )
+    , desktop_change_osd( 0 )
     , popup( 0 )
     , advanced_popup( 0 )
     , trans_popup( 0 )
@@ -125,9 +131,6 @@ Workspace::Workspace( bool restore )
     , global_shortcuts_disabled_for_client( false )
     , workspaceInit( true )
     , startup( 0 )
-    , layoutOrientation( Qt::Vertical )
-    , layoutX( -1 )
-    , layoutY( 2 )
     , managing_topmenus( false )
     , topmenu_selection( NULL )
     , topmenu_watcher( NULL )
@@ -139,6 +142,7 @@ Workspace::Workspace( bool restore )
     , cm_selection( NULL )
     , compositingSuspended( false )
     , compositeRate( 0 )
+    , xrrRefreshRate( 0 )
     , overlay( None )
     , overlay_visible( true )
     , overlay_shown( false )
@@ -155,6 +159,10 @@ Workspace::Workspace( bool restore )
     dbus.connect( QString(), "/KWin", "org.kde.KWin", "reinitCompositing",
         this, SLOT( slotReinitCompositing() ));
 
+    // Initialize desktop grid array
+    desktopGrid_[0] = 0;
+    desktopGrid_[1] = 0;
+
     _self = this;
     mgr = new PluginMgr;
     QX11Info info;
@@ -171,7 +179,9 @@ Workspace::Workspace( bool restore )
         this, SLOT( gotTemporaryRulesMessage(const QString&) ));
     connect( &rulesUpdatedTimer, SIGNAL( timeout() ), this, SLOT( writeWindowRules() ));
     connect( &unredirectTimer, SIGNAL( timeout() ), this, SLOT( delayedCheckUnredirect() ));
+    connect( &compositeResetTimer, SIGNAL( timeout()), this, SLOT( resetCompositing()));
     unredirectTimer.setSingleShot( true );
+    compositeResetTimer.setSingleShot( true );
 
     updateXTime(); // Needed for proper initialization of user_time in Client ctor
 
@@ -217,7 +227,7 @@ Workspace::Workspace( bool restore )
     client_keys = new KActionCollection( this );
     initShortcuts();
     tab_box = new TabBox( this );
-    popupinfo = new PopupInfo( this );
+    desktop_change_osd = new DesktopChangeOSD( this );
 
     init();
 
@@ -229,6 +239,7 @@ Workspace::Workspace( bool restore )
 
 void Workspace::init()
     {
+    reserveElectricBorderActions( true );
     if( options->electricBorders() == Options::ElectricAlways )
         reserveElectricBorderSwitching( true );
     updateElectricBorders();
@@ -332,6 +343,7 @@ void Workspace::init()
 
     loadDesktopSettings();
     updateDesktopLayout();
+    desktop_change_osd->numberDesktopsChanged();
     // Extra NETRootInfo instance in Client mode is needed to get the values of the properties
     NETRootInfo client_info( display(), NET::ActiveWindow | NET::CurrentDesktop );
     int initial_desktop;
@@ -385,6 +397,7 @@ void Workspace::init()
         Window root_return, parent_return;
         Window* wins;
         XQueryTree( display(), rootWindow(), &root_return, &parent_return, &wins, &nwins );
+        bool fixoffset = KCmdLineArgs::parsedArgs()->getOption( "crashes" ).toInt() > 0;
         for( i = 0; i < nwins; i++ )
             {
             XWindowAttributes attr;
@@ -397,7 +410,11 @@ void Workspace::init()
             if( topmenu_space && topmenu_space->winId() == wins[i] )
                 continue;
             if( attr.map_state != IsUnmapped )
+                {
+                if( fixoffset )
+                    fixPositionAfterCrash( wins[ i ], attr );
                 createClient( wins[i], true );
+                }
             }
         if( wins )
             XFree( (void*)( wins ));
@@ -408,8 +425,8 @@ void Workspace::init()
         updateClientArea();
 
         // NETWM spec says we have to set it to (0,0) if we don't support it
-        NETPoint* viewports = new NETPoint[number_of_desktops];
-        rootInfo->setDesktopViewport( number_of_desktops, *viewports );
+        NETPoint* viewports = new NETPoint[numberOfDesktops()];
+        rootInfo->setDesktopViewport( numberOfDesktops(), *viewports );
         delete[] viewports;
         QRect geom = Kephal::ScreenUtils::desktopGeometry();
         NETSize desktop_geometry;
@@ -469,7 +486,7 @@ Workspace::~Workspace()
         ++it )
         (*it)->release();
     delete tab_box;
-    delete popupinfo;
+    delete desktop_change_osd;
     discardPopup();
     XDeleteProperty( display(), rootWindow(), atoms->kwin_running );
 
@@ -496,6 +513,8 @@ Workspace::~Workspace()
 
     // TODO: ungrabXServer();
 
+    delete[] desktopGrid_;
+
     _self = 0;
     }
 
@@ -982,10 +1001,11 @@ void Workspace::reconfigure()
  */
 bool Workspace::waitForCompositingSetup()
     {
-    if( !reconfigureTimer.isActive() )
-        return false;
-    reconfigureTimer.stop();
-    slotReconfigure();
+    if( reconfigureTimer.isActive() )
+        {
+        reconfigureTimer.stop();
+        slotReconfigure();
+        }
     return compositingActive();
     }
 
@@ -1006,6 +1026,7 @@ void Workspace::slotReconfigure()
     kDebug( 1212 ) << "Workspace::slotReconfigure()";
     reconfigureTimer.stop();
 
+    reserveElectricBorderActions( false );
     if( options->electricBorders() == Options::ElectricAlways )
         reserveElectricBorderSwitching( false );
 
@@ -1013,7 +1034,7 @@ void Workspace::slotReconfigure()
     unsigned long changed = options->updateSettings();
 
     tab_box->reconfigure();
-    popupinfo->reconfigure();
+    desktop_change_osd->reconfigure();
     initPositioning->reinitCascading( 0 );
     readShortcuts();
     forEachClient( CheckIgnoreFocusStealingProcedure());
@@ -1038,9 +1059,10 @@ void Workspace::slotReconfigure()
         {
         forEachClient( CheckBorderSizesProcedure() );
         foreach( Client* c, clients )
-            c->repaintDecoration();
+            c->triggerDecorationRepaint();
         }
 
+    reserveElectricBorderActions( true );
     if( options->electricBorders() == Options::ElectricAlways )
         reserveElectricBorderSwitching( true );
     updateElectricBorders();
@@ -1096,9 +1118,14 @@ void Workspace::slotReinitCompositing()
 
     // Restart compositing
     finishCompositing();
+    // resume compositing if suspended
+    compositingSuspended = false;
     setupCompositing();
     if( effects ) // setupCompositing() may fail
+        {
         effects->reconfigure();
+        emit compositingToggled( true );
+        }
     }
 
 void Workspace::loadDesktopSettings()
@@ -1112,11 +1139,15 @@ void Workspace::loadDesktopSettings()
     KConfigGroup group( c, groupname );
 
     int n = group.readEntry( "Number", 4 );
-    number_of_desktops = n;
+    desktopCount_ = n;
     workarea.clear();
     workarea.resize( n + 1 );
+    restrictedmovearea.clear();
+    restrictedmovearea.resize( n + 1 );
+    oldrestrictedmovearea.clear();
+    oldrestrictedmovearea.resize( n + 1 );
     screenarea.clear();
-    rootInfo->setNumberOfDesktops( number_of_desktops );
+    rootInfo->setNumberOfDesktops( n );
     desktop_focus_chain.resize( n );
     // Make it +1, so that it can be accessed as [1..numberofdesktops]
     focus_chain.resize( n + 1 );
@@ -1138,8 +1169,8 @@ void Workspace::saveDesktopSettings()
         groupname.sprintf( "Desktops-screen-%d", screen_number );
     KConfigGroup group( c, groupname );
 
-    group.writeEntry( "Number", number_of_desktops );
-    for( int i = 1; i <= number_of_desktops; i++ )
+    group.writeEntry( "Number", numberOfDesktops() );
+    for( int i = 1; i <= numberOfDesktops(); i++ )
         {
         QString s = desktopName( i );
         QString defaultvalue = i18n( "Desktop %1", i );
@@ -1299,7 +1330,7 @@ ObscuringWindows::~ObscuringWindows()
  */
 bool Workspace::setCurrentDesktop( int new_desktop )
     {
-    if( new_desktop < 1 || new_desktop > number_of_desktops )
+    if( new_desktop < 1 || new_desktop > numberOfDesktops() )
         return false;
 
     closeActivePopup();
@@ -1307,8 +1338,8 @@ bool Workspace::setCurrentDesktop( int new_desktop )
     // TODO: Q_ASSERT( block_stacking_updates == 0 ); // Make sure stacking_order is up to date
     StackingUpdatesBlocker blocker( this );
 
-    int old_desktop = current_desktop;
-    if (new_desktop != current_desktop )
+    int old_desktop = currentDesktop();
+    if (new_desktop != currentDesktop() )
         {
         ++block_showing_desktop;
         // Optimized Desktop switching: unmapping done from back to front
@@ -1317,7 +1348,7 @@ bool Workspace::setCurrentDesktop( int new_desktop )
 
         ObscuringWindows obs_wins;
 
-        current_desktop = new_desktop; // Change the desktop (so that Client::updateVisibility() works)
+        currentDesktop_ = new_desktop; // Change the desktop (so that Client::updateVisibility() works)
 
         for( ClientList::ConstIterator it = stacking_order.constBegin();
             it != stacking_order.constEnd();
@@ -1330,7 +1361,7 @@ bool Workspace::setCurrentDesktop( int new_desktop )
                 }
 
         // Now propagate the change, after hiding, before showing
-        rootInfo->setCurrentDesktop( current_desktop );
+        rootInfo->setCurrentDesktop( currentDesktop() );
 
         if( movingClient && !movingClient->isOnDesktop( new_desktop ))
             movingClient->setDesktop( new_desktop );
@@ -1402,8 +1433,9 @@ bool Workspace::setCurrentDesktop( int new_desktop )
     //    s += QString::number( desktop_focus_chain[i] ) + ", ";
     //kDebug( 1212 ) << s << "}\n";
 
-    if( old_desktop != 0 )  // Not for the very first time
-        popupinfo->showInfo( desktopName( currentDesktop() ));
+    // Not for the very first time, only if something changed and there are more than 1 desktops
+    if( old_desktop != 0 && old_desktop != new_desktop && numberOfDesktops() > 1 )
+        desktop_change_osd->desktopChanged( old_desktop );
 
     if( effects != NULL && old_desktop != 0 && old_desktop != new_desktop )
         static_cast<EffectsHandlerImpl*>( effects )->desktopChanged( old_desktop );
@@ -1431,162 +1463,35 @@ void Workspace::previousDesktop()
     setCurrentDesktop( desktop > 0 ? desktop : numberOfDesktops() );
     }
 
-int Workspace::desktopToRight( int desktop, bool wrap ) const
-    {
-    int x,y;
-    Qt::Orientation orientation;
-    calcDesktopLayout( &x, &y, &orientation );
-    int dt = desktop - 1;
-    if( orientation == Qt::Vertical )
-        {
-        dt += y;
-        if( dt >= numberOfDesktops() )
-            {
-            if( wrap )
-                dt -= numberOfDesktops();
-            else
-                return desktop;
-            }
-        }
-    else
-        {
-        int d = ( dt % x ) + 1;
-        if( d >= x )
-            {
-            if( wrap )
-                d -= x;
-            else
-                return desktop;
-            }
-        dt = dt - ( dt % x ) + d;
-        }
-    return dt + 1;
-    }
-
-int Workspace::desktopToLeft( int desktop, bool wrap ) const
-    {
-    int x,y;
-    Qt::Orientation orientation;
-    calcDesktopLayout( &x, &y, &orientation );
-    int dt = desktop - 1;
-    if( orientation == Qt::Vertical )
-        {
-        dt -= y;
-        if( dt < 0 )
-            {
-            if( wrap )
-                dt += numberOfDesktops();
-            else
-                return desktop;
-            }
-        }
-    else
-        {
-        int d = ( dt % x ) - 1;
-        if( d < 0 )
-            {
-            if( wrap )
-                d += x;
-            else
-                return desktop;
-            }
-        dt = dt - ( dt % x ) + d;
-        }
-    return dt + 1;
-    }
-
-int Workspace::desktopUp( int desktop, bool wrap ) const
-    {
-    int x,y;
-    Qt::Orientation orientation;
-    calcDesktopLayout( &x, &y, &orientation);
-    int dt = desktop - 1;
-    if( orientation == Qt::Horizontal )
-        {
-        dt -= x;
-        if( dt < 0 )
-            {
-            if( wrap )
-                dt += numberOfDesktops();
-            else
-                return desktop;
-            }
-        }
-    else
-        {
-        int d = ( dt % y ) - 1;
-        if( d < 0 )
-            {
-            if( wrap )
-                d += y;
-            else
-                return desktop;
-            }
-        dt = dt - ( dt % y ) + d;
-        }
-    return dt + 1;
-    }
-
-int Workspace::desktopDown( int desktop, bool wrap ) const
-    {
-    int x,y;
-    Qt::Orientation orientation;
-    calcDesktopLayout( &x, &y, &orientation);
-    int dt = desktop - 1;
-    if( orientation == Qt::Horizontal )
-        {
-        dt += x;
-        if( dt >= numberOfDesktops() )
-            {
-            if( wrap )
-                dt -= numberOfDesktops();
-            else
-                return desktop;
-            }
-        }
-    else
-        {
-        int d = ( dt % y ) + 1;
-        if( d >= y )
-            {
-            if( wrap )
-                d -= y;
-            else
-                return desktop;
-            }
-        dt = dt - ( dt % y ) + d;
-        }
-    return dt + 1;
-    }
-
 /**
  * Sets the number of virtual desktops to \a n
  */
 void Workspace::setNumberOfDesktops( int n )
     {
-    if( n == number_of_desktops )
+    if( n == numberOfDesktops() )
         return;
-    int old_number_of_desktops = number_of_desktops;
-    number_of_desktops = n;
+    int old_number_of_desktops = numberOfDesktops();
+    desktopCount_ = n;
+    updateDesktopLayout(); // Make sure the layout is still valid
 
     if( currentDesktop() > numberOfDesktops() )
         setCurrentDesktop( numberOfDesktops() );
 
     // If increasing the number, do the resizing now, otherwise
     // after the moving of windows to still existing desktops
-    if( old_number_of_desktops < number_of_desktops )
+    if( old_number_of_desktops < numberOfDesktops() )
         {
-        rootInfo->setNumberOfDesktops( number_of_desktops );
-        NETPoint* viewports = new NETPoint[number_of_desktops];
-        rootInfo->setDesktopViewport( number_of_desktops, *viewports );
+        rootInfo->setNumberOfDesktops( numberOfDesktops() );
+        NETPoint* viewports = new NETPoint[numberOfDesktops()];
+        rootInfo->setDesktopViewport( numberOfDesktops(), *viewports );
         delete[] viewports;
         updateClientArea( true );
-        focus_chain.resize( number_of_desktops + 1 );
+        focus_chain.resize( numberOfDesktops() + 1 );
         }
 
     // If the number of desktops decreased, move all windows
     // that would be hidden to the last visible desktop
-    if( old_number_of_desktops > number_of_desktops )
+    if( old_number_of_desktops > numberOfDesktops() )
         {
         for( ClientList::ConstIterator it = clients.constBegin();
             it != clients.constEnd();
@@ -1594,14 +1499,14 @@ void Workspace::setNumberOfDesktops( int n )
             if( !(*it)->isOnAllDesktops() && (*it)->desktop() > numberOfDesktops() )
                 sendClientToDesktop( *it, numberOfDesktops(), true );
         }
-    if( old_number_of_desktops > number_of_desktops )
+    if( old_number_of_desktops > numberOfDesktops() )
         {
-        rootInfo->setNumberOfDesktops( number_of_desktops );
-        NETPoint* viewports = new NETPoint[number_of_desktops];
-        rootInfo->setDesktopViewport( number_of_desktops, *viewports );
+        rootInfo->setNumberOfDesktops( numberOfDesktops() );
+        NETPoint* viewports = new NETPoint[numberOfDesktops()];
+        rootInfo->setDesktopViewport( numberOfDesktops(), *viewports );
         delete[] viewports;
         updateClientArea( true );
-        focus_chain.resize( number_of_desktops + 1 );
+        focus_chain.resize( numberOfDesktops() + 1 );
         }
 
     saveDesktopSettings();
@@ -1610,6 +1515,9 @@ void Workspace::setNumberOfDesktops( int n )
     desktop_focus_chain.resize( n );
     for( int i = 0; i < int( desktop_focus_chain.size() ); i++ )
         desktop_focus_chain[i] = i+1;
+
+    // reset the desktop change osd
+    desktop_change_osd->numberDesktopsChanged();
     }
 
 /**
@@ -1723,35 +1631,6 @@ void Workspace::sendClientToScreen( Client* c, int screen )
         active_screen = screen;
     }
 
-void Workspace::updateDesktopLayout()
-    {
-    //rootInfo->desktopLayoutCorner(); // I don't find this worth bothering, feel free to
-    layoutOrientation = ( rootInfo->desktopLayoutOrientation() == NET::OrientationHorizontal
-        ? Qt::Horizontal : Qt::Vertical );
-    layoutX = rootInfo->desktopLayoutColumnsRows().width();
-    layoutY = rootInfo->desktopLayoutColumnsRows().height();
-    if( layoutX == 0 && layoutY == 0 ) // Not given, set default layout
-        layoutY = 2;
-    }
-
-void Workspace::calcDesktopLayout( int* xp, int* yp, Qt::Orientation* orientation ) const
-    {
-    int x = layoutX; // <= 0 means compute it from the other and total number of desktops
-    int y = layoutY;
-    if(( x <= 0 ) && ( y > 0 ))
-       x = ( numberOfDesktops() + y - 1 ) / y;
-    else if(( y <= 0) && ( x > 0 ))
-       y = ( numberOfDesktops() + x - 1 ) / x;
-
-    if( x <= 0 )
-       x = 1;
-    if( y <= 0 )
-       y = 1;
-    *xp = x;
-    *yp = y;
-    *orientation = layoutOrientation;
-    }
-
 void Workspace::killWindowId( Window window_to_kill )
     {
     if( window_to_kill == None )
@@ -2125,6 +2004,7 @@ void Workspace::updateElectricBorders()
     {
     electric_time_first = xTime();
     electric_time_last = xTime();
+    electric_time_last_trigger = xTime();
     electric_current_border = ElectricNone;
     QRect r = Kephal::ScreenUtils::desktopGeometry();
     electricTop = r.top();
@@ -2180,6 +2060,18 @@ void Workspace::destroyElectricBorders()
         }
     }
 
+void Workspace::reserveElectricBorderActions( bool reserve )
+    {
+    for( int pos = 0; pos < ELECTRIC_COUNT; ++pos )
+        if( options->electricBorderAction( static_cast<ElectricBorder>( pos )))
+            {
+            if( reserve )
+                reserveElectricBorder( static_cast<ElectricBorder>( pos ));
+            else
+                unreserveElectricBorder( static_cast<ElectricBorder>( pos ));
+            }
+    }
+
 void Workspace::reserveElectricBorderSwitching( bool reserve )
     {
     for( int pos = 0; pos < ELECTRIC_COUNT; ++pos )
@@ -2223,6 +2115,7 @@ void Workspace::checkElectricBorder(const QPoint& pos, Time now)
 
     Time treshold_set = options->electricBorderDelay(); // Set timeout
     Time treshold_reset = 250; // Reset timeout
+    Time treshold_trigger = options->electricBorderCooldown(); // Minimum time between triggers
     int distance_reset = 30; // Mouse should not move more than this many pixels
 
     ElectricBorder border;
@@ -2250,6 +2143,7 @@ void Workspace::checkElectricBorder(const QPoint& pos, Time now)
 
     if(( electric_current_border == border ) &&
        ( timestampDiff( electric_time_last, now ) < treshold_reset ) &&
+       ( timestampDiff( electric_time_last_trigger, now ) > treshold_trigger ) &&
        (( pos-electric_push_point ).manhattanLength() < distance_reset ))
         {
         electric_time_last = now;
@@ -2257,11 +2151,41 @@ void Workspace::checkElectricBorder(const QPoint& pos, Time now)
         if( timestampDiff( electric_time_first, now ) > treshold_set )
             {
             electric_current_border = ElectricNone;
-            if( effects && static_cast<EffectsHandlerImpl*>( effects )->borderActivated( border ))
-                {} // Handled by effects
-            else
+            electric_time_last_trigger = now;
+            if( movingClient )
+                { // If moving a client or have force doing the desktop switch
+                if( options->electricBorders() != Options::ElectricDisabled )
+                    electricBorderSwitchDesktop( border, pos );
+                return; // Don't reset cursor position
+                }
+            if( options->electricBorders() == Options::ElectricAlways &&
+              ( border == ElectricTop || border == ElectricRight ||
+                border == ElectricBottom || border == ElectricLeft ))
+                { // If desktop switching is always enabled don't apply it to the corners if
+                  // an effect is applied to it (We will check that later).
                 electricBorderSwitchDesktop( border, pos );
-            return;
+                return; // Don't reset cursor position
+                }
+            switch( options->electricBorderAction( border ))
+                {
+                case ElectricActionDashboard: // Display Plasma dashboard
+                    {
+                    QDBusInterface plasmaApp( "org.kde.plasma-desktop", "/App" );
+                    plasmaApp.call( "toggleDashboard" );
+                    }
+                    break;
+                case ElectricActionNone: // Either desktop switching or an effect
+                default:
+                    {
+                    if( effects && static_cast<EffectsHandlerImpl*>( effects )->borderActivated( border ))
+                        {} // Handled by effects
+                    else
+                        {
+                        electricBorderSwitchDesktop( border, pos );
+                        return; // Don't reset cursor position
+                        }
+                    }
+                }
             }
         }
     else
@@ -2296,12 +2220,12 @@ void Workspace::electricBorderSwitchDesktop( ElectricBorder border, const QPoint
         }
     if( border == ElectricTop || border == ElectricTopLeft || border == ElectricTopRight )
         {
-        desk = desktopUp( desk, options->rollOverDesktops );
+        desk = desktopAbove( desk, options->rollOverDesktops );
         pos.setY( displayHeight() - 1 - OFFSET );
         }
     if( border == ElectricBottom || border == ElectricBottomLeft || border == ElectricBottomRight )
         {
-        desk = desktopDown( desk, options->rollOverDesktops );
+        desk = desktopBelow( desk, options->rollOverDesktops );
         pos.setY( OFFSET );
         }
     int desk_before = currentDesktop();
@@ -2433,7 +2357,7 @@ void Workspace::setupTopMenuHandling()
 int Workspace::topMenuHeight() const
     {
     if( topmenu_height == 0 )
-        { // Simply create a dummy menubar and use its preffered height as the menu height
+        { // Simply create a dummy menubar and use its preferred height as the menu height
         KMenuBar tmpmenu;
         tmpmenu.addAction( "dummy" );
         topmenu_height = tmpmenu.sizeHint().height();
