@@ -4,6 +4,7 @@
 
 Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
+Copyright (C) 2009 Lucas Murray <lmurray@undefinedfire.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -55,7 +56,7 @@ namespace KWin
 
 class Client;
 class TabBox;
-class PopupInfo;
+class DesktopChangeOSD;
 class RootInfo;
 class PluginMgr;
 class Placement;
@@ -89,6 +90,9 @@ class Workspace : public QObject, public KDecorationDefines
         QRect clientArea( clientAreaOption, const Client* c ) const;
         QRect clientArea( clientAreaOption, int screen, int desktop ) const;
 
+        QRegion restrictedMoveArea( int desktop, StrutAreas areas = StrutAreaAll ) const;
+        QRegion previousRestrictedMoveArea( int desktop, StrutAreas areas = StrutAreaAll ) const;
+
         /**
          * @internal
          */
@@ -155,23 +159,130 @@ class Workspace : public QObject, public KDecorationDefines
         void checkElectricBorder(const QPoint& pos, Time time);
         void reserveElectricBorder( ElectricBorder border );
         void unreserveElectricBorder( ElectricBorder border );
+        void reserveElectricBorderActions( bool reserve );
         void reserveElectricBorderSwitching( bool reserve );
 
+        //-------------------------------------------------
+        // Desktop layout
+
+    public:
         /**
-         * Returns the current virtual desktop of this workspace
+         * @returns Total number of desktops currently in existance.
+         */
+        int numberOfDesktops() const;
+        /**
+         * Set the number of available desktops to @a count. This function overrides any previous
+         * grid layout.
+         */
+        void setNumberOfDesktops( int count );
+        /**
+         * Called from within setNumberOfDesktops() to ensure the desktop layout is still valid.
+         */
+        void updateDesktopLayout();
+
+        /**
+         * @returns The size of desktop layout in grid units.
+         */
+        QSize desktopGridSize() const;
+        /**
+         * @returns The width of desktop layout in grid units.
+         */
+        int desktopGridWidth() const;
+        /**
+         * @returns The height of desktop layout in grid units.
+         */
+        int desktopGridHeight() const;
+        /**
+         * @returns The width of desktop layout in pixels. Equivalent to gridWidth() *
+         * ::displayWidth().
+         */
+        int workspaceWidth() const;
+        /**
+         * @returns The height of desktop layout in pixels. Equivalent to gridHeight() *
+         * ::displayHeight().
+         */
+        int workspaceHeight() const;
+
+        /**
+         * @returns The ID of the current desktop.
          */
         int currentDesktop() const;
         /**
-         * Returns the number of virtual desktops of this workspace
+         * Set the current desktop to @a current.
+         * @returns True on success, false otherwise.
          */
-        int numberOfDesktops() const;
-        void setNumberOfDesktops( int n );
-        void calcDesktopLayout( int* x, int* y, Qt::Orientation* orientation ) const;
-        int desktopToRight( int desktop, bool wrap ) const;
-        int desktopToLeft( int desktop, bool wrap ) const;
-        int desktopUp( int desktop, bool wrap ) const;
-        int desktopDown( int desktop, bool wrap ) const;
+        bool setCurrentDesktop( int current );
 
+        /**
+         * Generate a desktop layout from EWMH _NET_DESKTOP_LAYOUT property parameters.
+         */
+        void setNETDesktopLayout( Qt::Orientation orientation, int width, int height, int startingCorner );
+
+        /**
+         * @returns The ID of the desktop at the point @a coords or 0 if no desktop exists at that
+         * point. @a coords is to be in grid units.
+         */
+        int desktopAtCoords( QPoint coords ) const;
+        /**
+         * @returns The coords of desktop @a id in grid units.
+         */
+        QPoint desktopGridCoords( int id ) const;
+        /**
+         * @returns The coords of the top-left corner of desktop @a id in pixels.
+         */
+        QPoint desktopCoords( int id ) const;
+
+        /**
+         * @returns The ID of the desktop above desktop @a id. Wraps around to the bottom of
+         * the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        int desktopAbove( int id = 0, bool wrap = true ) const;
+        /**
+         * @returns The ID of the desktop to the right of desktop @a id. Wraps around to the
+         * left of the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        int desktopToRight( int id = 0, bool wrap = true ) const;
+        /**
+         * @returns The ID of the desktop below desktop @a id. Wraps around to the top of the
+         * layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        int desktopBelow( int id = 0, bool wrap = true ) const;
+        /**
+         * @returns The ID of the desktop to the left of desktop @a id. Wraps around to the
+         * right of the layout if @a wrap is set. If @a id is not set use the current one.
+         */
+        int desktopToLeft( int id = 0, bool wrap = true ) const;
+
+        /**
+         * @returns Whether or not the layout is allowed to be modified by the user.
+         */
+        bool isDesktopLayoutDynamic() const;
+        /**
+         * Sets whether or not this layout can be modified by the user.
+         */
+        void setDesktopLayoutDynamicity( bool dynamicity );
+        /**
+         * Create new desktop at the point @a coords
+         * @returns The ID of the created desktop
+         */
+        int addDesktop( QPoint coords );
+        /**
+         * Deletes the desktop with the ID @a id. All desktops with an ID greater than the one that
+         * was deleted will have their IDs' decremented.
+         */
+        void deleteDesktop( int id );
+
+    private:
+        int desktopCount_;
+        QSize desktopGridSize_;
+        int* desktopGrid_;
+        int currentDesktop_;
+        bool desktopLayoutDynamicity_;
+
+        //-------------------------------------------------
+        // Unsorted
+
+    public:
         int activeScreen() const;
         int numScreens() const;
         void checkActiveScreen( const Client* c );
@@ -217,6 +328,7 @@ class Workspace : public QObject, public KDecorationDefines
         // KDE4 remove me - And it's also in the DCOP interface :(
         void showWindowMenuAt( unsigned long id, int x, int y );
 
+        void toggleCompositing();
         void loadEffect( const QString& name );
         void toggleEffect( const QString& name );
         void reconfigureEffect( const QString& name );
@@ -253,19 +365,13 @@ class Workspace : public QObject, public KDecorationDefines
         bool rulesUpdatesDisabled() const;
 
         bool hasDecorationShadows() const;
-        QList< QList<QImage> > decorationShadowTextures();
-        int decorationShadowTextureList( ShadowType type ) const;
-        QList<QRect> decorationShadowQuads( ShadowType type, QSize size ) const;
-        double decorationShadowOpacity( ShadowType type ) const;
-        double decorationShadowBrightness( ShadowType type ) const;
-        double decorationShadowSaturation( ShadowType type ) const;
+        bool decorationHasAlpha() const;
 
         // D-Bus interface
         void cascadeDesktop();
         void unclutterDesktop();
         void doNotManage( const QString& );
         QList<int> decorationSupportedColors() const;
-        bool setCurrentDesktop( int new_desktop );
         void nextDesktop();
         void previousDesktop();
         void circulateDesktopApplications();
@@ -275,7 +381,6 @@ class Workspace : public QObject, public KDecorationDefines
         void setCurrentScreen( int new_screen );
 
         QString desktopName( int desk ) const;
-        void updateDesktopLayout();
         void setShowingDesktop( bool showing );
         void resetShowingDesktop( bool keep_hidden );
         bool showingDesktop() const;
@@ -361,6 +466,8 @@ class Workspace : public QObject, public KDecorationDefines
         void startMousePolling();
         void stopMousePolling();
 
+        void raiseElectricBorderWindows();
+
     public slots:
         void addRepaintFull();
         void refresh();
@@ -463,6 +570,13 @@ class Workspace : public QObject, public KDecorationDefines
         void slotWalkThroughWindows();
         void slotWalkBackThroughWindows();
 
+        void slotWalkThroughDesktopsKeyChanged( const QKeySequence& seq );
+        void slotWalkBackThroughDesktopsKeyChanged( const QKeySequence& seq );
+        void slotWalkThroughDesktopListKeyChanged( const QKeySequence& seq );
+        void slotWalkBackThroughDesktopListKeyChanged( const QKeySequence& seq );
+        void slotWalkThroughWindowsKeyChanged( const QKeySequence& seq );
+        void slotWalkBackThroughWindowsKeyChanged( const QKeySequence& seq );
+
         void slotWindowOperations();
         void slotWindowClose();
         void slotWindowMove();
@@ -488,6 +602,7 @@ class Workspace : public QObject, public KDecorationDefines
         void reconfigure();
         void slotReconfigure();
         void slotReinitCompositing();
+        void resetCompositing();
 
         void slotKillWindow();
 
@@ -520,6 +635,7 @@ class Workspace : public QObject, public KDecorationDefines
         void slotReloadConfig();
         void setPopupClientOpacity( QAction* action );
         void setupCompositing();
+        void finishCompositing();
         void performCompositing();
         void performMousePoll();
         void lostCMSelection();
@@ -530,6 +646,9 @@ class Workspace : public QObject, public KDecorationDefines
     protected:
         bool keyPressMouseEmulation( XKeyEvent& ev );
 
+    Q_SIGNALS:
+        Q_SCRIPTABLE void compositingToggled( bool active );
+
     private:
         void init();
         void initShortcuts();
@@ -565,6 +684,7 @@ class Workspace : public QObject, public KDecorationDefines
         void setupTopMenuHandling();
         void updateTopMenuGeometry( Client* c = NULL );
         void updateToolWindows( bool also_hide );
+        void fixPositionAfterCrash( Window w, const XWindowAttributes& attr );
 
         /// This is the right way to create a new client
         Client* createClient( Window w, bool is_mapped );
@@ -604,13 +724,10 @@ class Workspace : public QObject, public KDecorationDefines
 
         void updateClientArea( bool force );
 
-        void finishCompositing();
         bool windowRepaintsPending() const;
         void setCompositeTimer();
         void checkCompositePaintTime( int msec );
 
-        int current_desktop;
-        int number_of_desktops;
         QVector<int> desktop_focus_chain;
 
         QWidget* active_popup;
@@ -681,7 +798,7 @@ class Workspace : public QObject, public KDecorationDefines
         int block_focus;
 
         TabBox* tab_box;
-        PopupInfo* popupinfo;
+        DesktopChangeOSD* desktop_change_osd;
 
         QMenu* popup;
         QMenu* advanced_popup;
@@ -739,16 +856,17 @@ class Workspace : public QObject, public KDecorationDefines
         int electricBottom;
         Time electric_time_first;
         Time electric_time_last;
+        Time electric_time_last_trigger;
         QPoint electric_push_point;
         int electric_reserved[ELECTRIC_COUNT]; // Corners/edges used by something
 
-        Qt::Orientation layoutOrientation;
-        int layoutX;
-        int layoutY;
-
         Placement* initPositioning;
 
         QVector<QRect> workarea; // Array of workareas for virtual desktops
+        // Array of restricted areas that window cannot be moved into
+        QVector<StrutRects> restrictedmovearea;
+        // Array of the previous restricted areas that window cannot be moved into
+        QVector<StrutRects> oldrestrictedmovearea;
         QVector< QVector<QRect> > screenarea; // Array of workareas per xinerama screen for all virtual desktops
 
         bool managing_topmenus;
@@ -772,6 +890,7 @@ class Workspace : public QObject, public KDecorationDefines
         QTime nextPaintReference;
         QTimer mousePollingTimer;
         int compositeRate;
+        int xrrRefreshRate; // used only for compositing
         QRegion repaints_region;
         Window overlay; // XComposite overlay window
         bool overlay_visible;
@@ -782,6 +901,7 @@ class Workspace : public QObject, public KDecorationDefines
         QTimer unredirectTimer;
         bool forceUnredirectCheck;
         QList< int > composite_paint_times;
+        QTimer compositeResetTimer; // for compressing composite resets
 
     private:
         friend bool performTransiencyCheck();
@@ -830,6 +950,61 @@ class RootInfo : public NETRootInfo
         Workspace* workspace;
     };
 
+//---------------------------------------------------------
+// Desktop layout
+
+inline int Workspace::numberOfDesktops() const
+    {
+    return desktopCount_;
+    }
+
+inline QSize Workspace::desktopGridSize() const
+    {
+    return desktopGridSize_;
+    }
+
+inline int Workspace::desktopGridWidth() const
+    {
+    return desktopGridSize_.width();
+    }
+
+inline int Workspace::desktopGridHeight() const
+    {
+    return desktopGridSize_.height();
+    }
+
+inline int Workspace::workspaceWidth() const
+    {
+    return desktopGridSize_.width() * displayWidth();
+    }
+
+inline int Workspace::workspaceHeight() const
+    {
+    return desktopGridSize_.height() * displayHeight();
+    }
+
+inline int Workspace::currentDesktop() const
+    {
+    return currentDesktop_;
+    }
+
+inline int Workspace::desktopAtCoords( QPoint coords ) const
+    {
+    return desktopGrid_[coords.y() * desktopGridSize_.width() + coords.x()];
+    }
+
+inline bool Workspace::isDesktopLayoutDynamic() const
+    {
+    return desktopLayoutDynamicity_;
+    }
+
+inline void Workspace::setDesktopLayoutDynamicity( bool dynamicity )
+    {
+    desktopLayoutDynamicity_ = dynamicity;
+    }
+
+//---------------------------------------------------------
+// Unsorted
 
 inline bool Workspace::initializing() const
     {
@@ -846,16 +1021,6 @@ inline Client* Workspace::mostRecentlyActivatedClient() const
     return should_get_focus.count() > 0 ? should_get_focus.last() : active_client;
     }
 
-inline int Workspace::currentDesktop() const
-    {
-    return current_desktop;
-    }
-
-inline int Workspace::numberOfDesktops() const
-    {
-    return number_of_desktops;
-    }
-
 inline void Workspace::addGroup( Group* group, allowed_t )
     {
     groups.append( group );
@@ -1015,49 +1180,12 @@ inline void Workspace::checkCompositeTimer()
 
 inline bool Workspace::hasDecorationShadows() const
     {
-    return mgr->factory()->supports( AbilityCompositingShadow );
+    return mgr->factory()->supports( AbilityProvidesShadow );
     }
 
-inline QList< QList<QImage> > Workspace::decorationShadowTextures()
+inline bool Workspace::decorationHasAlpha() const
     {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast<KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowTextures();
-    return QList< QList<QImage> >();
-    }
-
-inline int Workspace::decorationShadowTextureList( ShadowType type ) const
-    {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast<KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowTextureList( type );
-    return -1;
-    }
-
-inline QList<QRect> Workspace::decorationShadowQuads( ShadowType type, QSize size ) const
-    {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast<KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowQuads( type, size );
-    return QList<QRect>();
-    }
-
-inline double Workspace::decorationShadowOpacity( ShadowType type ) const
-    {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast<KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowOpacity( type );
-    return 1.0;
-    }
-
-inline double Workspace::decorationShadowBrightness( ShadowType type ) const
-    {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast<KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowBrightness( type );
-    return 1.0;
-    }
-
-inline double Workspace::decorationShadowSaturation( ShadowType type ) const
-    {
-    if( KDecorationFactoryUnstable* factory = dynamic_cast< KDecorationFactoryUnstable*>( mgr->factory() ))
-        return factory->shadowSaturation( type );
-    return 1.0;
+    return mgr->factory()->supports( AbilityUsesAlphaChannel );
     }
 
 } // namespace
