@@ -32,15 +32,16 @@
 #endif
 #include <solid/networking.h>
 
-// Local
-#include "core/models.h"
-
 using namespace Kickoff;
 
 class SearchModel::Private
 {
 public:
-    Private(SearchModel *parent) : q(parent) {}
+    Private(SearchModel *parent)
+        : q(parent),
+          displayOrder(NameAfterDescription)
+    {
+    }
 
     void addItemForIface(SearchInterface *iface, QStandardItem *item) {
         int index = searchIfaces.indexOf(iface);
@@ -55,6 +56,7 @@ public:
 
     SearchModel * const q;
     QList<SearchInterface*> searchIfaces;
+    DisplayOrder displayOrder;
 };
 
 SearchModel::SearchModel(QObject *parent)
@@ -65,7 +67,7 @@ SearchModel::SearchModel(QObject *parent)
     //d->searchIfaces << new IndexerSearch(this);
     d->searchIfaces << new WebSearch(this);
 
-    foreach(SearchInterface *iface, d->searchIfaces) {
+    foreach (SearchInterface *iface, d->searchIfaces) {
         QStandardItem *ifaceItem = new QStandardItem(iface->name());
         appendRow(ifaceItem);
         connect(iface, SIGNAL(resultsAvailable(QStringList)),
@@ -78,35 +80,39 @@ SearchModel::SearchModel(QObject *parent)
                 this, SIGNAL(resultsAvailable()));
     }
 }
+
 SearchModel::~SearchModel()
 {
     delete d;
 }
+
 void SearchModel::resultsAvailable(const QStringList& results)
 {
     SearchInterface *iface = qobject_cast<SearchInterface*>(sender());
 
     Q_ASSERT(iface);
 
-    foreach(const QString& result, results) {
+    foreach (const QString& result, results) {
         //kDebug() << "Search hit from" << iface->name() << result;
-        QStandardItem *resultItem = StandardItemFactory::createItemForUrl(result);
+        QStandardItem *resultItem = StandardItemFactory::createItemForUrl(result, d->displayOrder);
         d->addItemForIface(iface, resultItem);
     }
 }
+
 void SearchModel::resultsAvailable(const ResultList& results)
 {
     SearchInterface *iface = qobject_cast<SearchInterface*>(sender());
 
     Q_ASSERT(iface);
 
-    foreach(const SearchResult& result, results) {
-        QStandardItem *item = StandardItemFactory::createItemForUrl(result.url);
+    foreach (const SearchResult& result, results) {
+        QStandardItem *item = StandardItemFactory::createItemForUrl(result.url, d->displayOrder);
         item->setData(result.title, Qt::DisplayRole);
         item->setData(result.subTitle, SubTitleRole);
         d->addItemForIface(iface, item);
     }
 }
+
 void SearchModel::setQuery(const QString& query)
 {
     d->clear();
@@ -115,18 +121,28 @@ void SearchModel::setQuery(const QString& query)
         return;
     }
 
-    foreach(SearchInterface *iface, d->searchIfaces) {
+    foreach (SearchInterface *iface, d->searchIfaces) {
         iface->setQuery(query);
     }
 }
 
+void SearchModel::setNameDisplayOrder(DisplayOrder displayOrder) 
+{
+    d->displayOrder = displayOrder;
+}
+
+DisplayOrder SearchModel::nameDisplayOrder() const
+{
+   return d->displayOrder;
+}
+
 SearchInterface::SearchInterface(QObject *parent)
-        : QObject(parent)
+    : QObject(parent)
 {
 }
 
 ApplicationSearch::ApplicationSearch(QObject *parent)
-        : SearchInterface(parent)
+    : SearchInterface(parent)
 {
 }
 
@@ -137,11 +153,7 @@ QString ApplicationSearch::name() const
 
 void ApplicationSearch::setQuery(const QString& query)
 {
-    //QString mimeName = mimeNameForQuery(query);
-    QString traderQuery = QString("((exist GenericName) and ('%1' ~~ GenericName)) or ('%1' ~~ Name) or ('%1' ~~ Exec) or ((exist Keywords) and ('%1' ~in Keywords))"
-                                  //" or ('%2' in MimeType)"
-                                 )
-                          .arg(query); //.arg(mimeName);
+    QString traderQuery = QString("((exist GenericName) and ('%1' ~~ GenericName)) or ('%1' ~~ Name) or ('%1' ~~ Exec) or ((exist Keywords) and ('%1' ~in Keywords))").arg(query);
     KServiceTypeTrader *trader = KServiceTypeTrader::self();
     KService::List results = trader->query("Application", traderQuery);
 
@@ -149,40 +161,33 @@ void ApplicationSearch::setQuery(const QString& query)
     // KDE 4 version
     QHash<QString, int> desktopNames;
     QSet<QString> execFields;
-
-
-    for (int i = 0;i < results.count();i++) {
+    for (int i = 0; i < results.count(); ++i) {
         KService::Ptr service = results[i];
         int existingPos = desktopNames.value(service->name(), -1);
         KService::Ptr existing = existingPos < 0 ? KService::Ptr(0) : results[existingPos];
 
-
-        if (!existing.isNull()) {
-            if (isLaterVersion(existing, service)) {
-                results[i] = 0;
-            } else if (isLaterVersion(service, existing)) {
-                results[existingPos] = 0;
-            } else {
-                // do not show more than one entry which does the same thing when run
-                // (ie. ignore entries that have an identical 'Exec' field to an existing
-                // entry)
-                if (execFields.contains(service->exec()) && service->noDisplay()) {
-                    results[i] = 0;
-                }
-            }
-        } else {
+        if (execFields.contains(service->exec()) || service->noDisplay()) {
+            // do not show more than one entry which does the same thing when run
+            // (ie. ignore entries that have an identical 'Exec' field to an existing
+            // entry)
+            results[i] = 0;
+        } else if (existing.isNull()) {
             desktopNames.insert(service->name(), i);
             execFields.insert(service->exec());
+        } else if (isLaterVersion(existing, service)) {
+            results[i] = 0;
+        } else if (isLaterVersion(service, existing)) {
+            results[existingPos] = 0;
         }
     }
 
-
     QStringList pathResults;
-    foreach(const KService::Ptr &service, results) {
+    foreach (const KService::Ptr &service, results) {
         if (!service.isNull() && !service->noDisplay())  {
             pathResults << service->entryPath();
         }
     }
+
     emit resultsAvailable(pathResults);
 }
 
@@ -195,14 +200,17 @@ QString ApplicationSearch::mimeNameForQuery(const QString& query) const
     }
     return QString();
 }
+
 WebSearch::WebSearch(QObject *parent)
         : SearchInterface(parent)
 {
 }
+
 QString WebSearch::name() const
 {
     return i18n("Web Searches");
 }
+
 void WebSearch::setQuery(const QString& query)
 {
     ResultList results;
@@ -212,14 +220,17 @@ void WebSearch::setQuery(const QString& query)
     results << googleResult;
     emit resultsAvailable(results);
 }
+
 IndexerSearch::IndexerSearch(QObject *parent)
         : SearchInterface(parent)
 {
 }
+
 QString IndexerSearch::name() const
 {
     return i18n("Documents");
 }
+
 void IndexerSearch::setQuery(const QString& query)
 {
 #ifdef HAVE_STRIGIDBUS
@@ -227,7 +238,7 @@ void IndexerSearch::setQuery(const QString& query)
 
     QList<QString> urls;
     QList<StrigiHit> hits = searchClient.getHits(query, 10, 0);
-    foreach(const StrigiHit& hit, hits) {
+    foreach (const StrigiHit& hit, hits) {
         if (!hit.uri.isEmpty()) {
             urls << hit.uri;
         }
