@@ -1,4 +1,4 @@
-/*  
+/*
     Copyright 2007 Robert Knight <robertknight@gmail.com>
 
     This library is free software; you can redistribute it and/or
@@ -36,6 +36,9 @@
 #include <KLineEdit>
 #include <KLocalizedString>
 
+//Plasma
+#include <Plasma/Theme>
+
 #include "ui/itemdelegate.h"
 
 using namespace Kickoff;
@@ -43,54 +46,67 @@ using namespace Kickoff;
 class SearchBar::Private
 {
 public:
-    Private() : editWidget(0),timer(0) {}
+    Private() : editWidget(0), timer(0) {}
 
     KLineEdit *editWidget;
+    QLabel *searchLabel;
     QTimer *timer;
 };
 
 SearchBar::SearchBar(QWidget *parent)
- : QWidget(parent)
- , d(new Private)
+        : QWidget(parent)
+        , d(new Private)
 {
     // timer for buffered updates
     d->timer = new QTimer(this);
     d->timer->setInterval(300);
     d->timer->setSingleShot(true);
-    connect(d->timer,SIGNAL(timeout()),this,SLOT(updateTimerExpired()));
-    connect(this,SIGNAL(startUpdateTimer()),d->timer,SLOT(start()));
+    connect(d->timer, SIGNAL(timeout()), this, SLOT(updateTimerExpired()));
+    connect(this, SIGNAL(startUpdateTimer()), d->timer, SLOT(start()));
 
     // setup UI
     QHBoxLayout *layout = new QHBoxLayout;
     layout->setMargin(3);
     layout->setSpacing(0); // we do the spacing manually to line up with the views below
 
-    QLabel *searchLabel = new QLabel(i18n("Search:"),this);
+    d->searchLabel = new QLabel(i18n("Search:"), this);
     QLabel *searchIcon = new QLabel(this);
 
-    QFileInfo fi(QDir(QDir::homePath()), ".face.icon");
+    const QFileInfo fi(QDir(QDir::homePath()), ".face.icon");
     if (fi.exists()) {
         searchIcon->setPixmap(QPixmap(fi.absoluteFilePath()).scaled(KIconLoader::SizeMedium, KIconLoader::SizeMedium, Qt::KeepAspectRatio));
-    }
-    else {
+    } else {
         searchIcon->setPixmap(KIcon("system-search").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
     }
 
     d->editWidget = new KLineEdit(this);
     d->editWidget->installEventFilter(this);
     d->editWidget->setClearButtonShown(true);
-    connect(d->editWidget,SIGNAL(textChanged(QString)),this,SIGNAL(startUpdateTimer()));
+    connect(d->editWidget, SIGNAL(textChanged(QString)), this, SIGNAL(startUpdateTimer()));
 
     //add arbitrary spacing
     layout->addSpacing(2);
     layout->addWidget(searchIcon);
     layout->addSpacing(5);
-    layout->addWidget(searchLabel);
+    layout->addWidget(d->searchLabel);
     layout->addSpacing(5);
     layout->addWidget(d->editWidget);
     setLayout(layout);
 
     setFocusProxy(d->editWidget);
+
+    updateThemedPalette();
+    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
+            this, SLOT(updateThemedPalette()));
+}
+
+void SearchBar::updateThemedPalette()
+{
+    QColor color = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
+    QPalette p = d->searchLabel->palette();
+    p.setColor(QPalette::Normal, QPalette::WindowText, color);
+    p.setColor(QPalette::Inactive, QPalette::WindowText, color);
+    d->searchLabel->setPalette(p);
 }
 
 void SearchBar::updateTimerExpired()
@@ -103,7 +119,7 @@ SearchBar::~SearchBar()
     delete d;
 }
 
-bool SearchBar::eventFilter(QObject *watched,QEvent *event)
+bool SearchBar::eventFilter(QObject *watched, QEvent *event)
 {
     // left and right arrow key presses in the search edit when the
     // edit is empty are propagated up to the parent widget
@@ -112,20 +128,12 @@ bool SearchBar::eventFilter(QObject *watched,QEvent *event)
     if (watched == d->editWidget && event->type() == QEvent::KeyPress) {
         QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
         if ((keyEvent->key() == Qt::Key_Left || keyEvent->key() == Qt::Key_Right) &&
-            d->editWidget->text().isEmpty()) {
-            QCoreApplication::sendEvent(this,event);
+                d->editWidget->text().isEmpty()) {
+            QCoreApplication::sendEvent(this, event);
             return true;
         }
     }
-    return false; 
-}
-
-void SearchBar::paintEvent(QPaintEvent *event)
-{
-    Q_UNUSED(event);
-    QPainter p(this);
-    p.setPen(QPen(palette().mid(), 1));
-    p.drawLine(0, height() - 1, width() - 1, height() - 1);
+    return false;
 }
 
 void SearchBar::clear()
