@@ -81,19 +81,13 @@ void Hdd::createConfigurationInterface(KConfigDialog *parent)
     ui.treeView->setModel(&m_hddModel);
     ui.treeView->resizeColumnToContents(0);
     ui.intervalSpinBox->setValue(interval() / 60 / 1000);
-    updateSpinBoxSuffix(interval() / 60 / 1000);
+    ui.intervalSpinBox->setSuffix(ki18np(" minute", " minutes"));
 
     parent->addPage(widget, i18n("Partitions"), "drive-harddisk");
-    connect(ui.intervalSpinBox, SIGNAL(valueChanged()), this, SLOT(ui.intervalSpinBox(int)));
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
 }
 
-void Hdd::updateSpinBoxSuffix(int interval)
-{
-    ui.intervalSpinBox->setSuffix(i18np(" minute", " minutes", interval));
-}
-
 void Hdd::configAccepted()
 {
     KConfigGroup cg = config();
@@ -265,8 +259,8 @@ bool Hdd::isValidDevice(const QString& uuid, Plasma::DataEngine::Data* data)
                             data->value("File System Type").toString() <<
                             data->value("Size").toString();
         */
-        if ((data->value("Usage").toString() != "File System" &&
-             data->value("Usage").toString() != "Raid") ||
+        if ((data->value("Usage").toString() != i18n("File System") &&
+             data->value("Usage").toString() != i18n("Raid")) ||
             data->value("File System Type").toString() == "swap") {
             QStringList list = items();
             list.removeAll(uuid);
