@@ -59,7 +59,7 @@
 #include "tasks.h"
 #include "taskgroupitem.h"
 
-static const int HOVER_EFFECT_TIMEOUT = 800;
+static const int HOVER_EFFECT_TIMEOUT = 900;
 
 AbstractTaskItem::AbstractTaskItem(QGraphicsWidget *parent, Tasks *applet)
     : QGraphicsWidget(parent),
@@ -164,6 +164,7 @@ void AbstractTaskItem::clearAbstractItem()
 void AbstractTaskItem::setText(const QString &text)
 {
     m_text = text;
+    m_cachedShadow = QPixmap();
 }
 
 void AbstractTaskItem::setIcon(const QIcon &icon)
@@ -283,7 +284,8 @@ void AbstractTaskItem::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
 {
     Q_UNUSED(event)
     fadeBackground("hover", 175, true);
-    if (parentGroup()) {
+    QGraphicsWidget *w = parentWidget();
+    if (w && this != m_applet->rootGroupItem()) {
         if (m_hoverEffectTimerId) {
             killTimer(m_hoverEffectTimerId);
         }
@@ -462,9 +464,10 @@ void AbstractTaskItem::timerEvent(QTimerEvent *event)
             data.resize(actualCount);
         }
 
-        if (!data.isEmpty() && m_applet->view()) {
+        QGraphicsView *view = m_applet->view();
+        if (!data.isEmpty() && view) {
             Display *dpy = QX11Info::display();
-            const WId winId = m_applet->view()->winId();
+            const WId winId = view->winId();
             Atom atom = XInternAtom(dpy, "_KDE_WINDOW_HIGHLIGHT", False);
             XChangeProperty(dpy, winId, atom, atom, 32, PropModeReplace,
                             reinterpret_cast<unsigned char *>(data.data()), data.size());
@@ -507,6 +510,34 @@ void AbstractTaskItem::syncActiveRect()
     itemBackground->setElementPrefix(m_backgroundPrefix);
 }
 
+void AbstractTaskItem::resizeEvent(QGraphicsSceneResizeEvent *event)
+{
+    m_cachedShadow = QPixmap();
+
+    syncActiveRect();
+
+    resizeBackground(event->newSize().toSize());
+}
+
+void AbstractTaskItem::resizeBackground(const QSize &size)
+{
+    Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
+
+    itemBackground->setElementPrefix("focus");
+    m_applet->resizeItemBackground(size);
+    itemBackground->setElementPrefix("normal");
+    m_applet->resizeItemBackground(size);
+    itemBackground->setElementPrefix("minimized");
+    m_applet->resizeItemBackground(size);
+    itemBackground->setElementPrefix("attention");
+    m_applet->resizeItemBackground(size);
+    itemBackground->setElementPrefix("hover");
+    m_applet->resizeItemBackground(size);
+
+    //restore the prefix
+    itemBackground->setElementPrefix(m_backgroundPrefix);
+}
+
 void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *)
 {
     // Do not paint with invalid sizes, the happens when the layout is being initialized
@@ -520,23 +551,9 @@ void AbstractTaskItem::drawBackground(QPainter *painter, const QStyleOptionGraph
     */
     Plasma::FrameSvg *itemBackground = m_applet->itemBackground();
 
-    //if the size is changed have to resize all the elements
-    if (itemBackground->size() != size().toSize() && itemBackground->size() != m_activeRect.size().toSize()) {
-        syncActiveRect();
-
-        itemBackground->setElementPrefix("focus");
-        m_applet->resizeItemBackground(m_activeRect.size().toSize());
-        itemBackground->setElementPrefix("normal");
-        m_applet->resizeItemBackground(size().toSize());
-        itemBackground->setElementPrefix("minimized");
-        m_applet->resizeItemBackground(size().toSize());
-        itemBackground->setElementPrefix("attention");
-        m_applet->resizeItemBackground(size().toSize());
-        itemBackground->setElementPrefix("hover");
-        m_applet->resizeItemBackground(m_activeRect.size().toSize());
-
-        //restore the prefix
-        itemBackground->setElementPrefix(m_backgroundPrefix);
+    //since a single framesvg is shared between all tasks, we could have to resize it even if there wasn't a resizeevent
+    if (size().toSize() != itemBackground->frameSize()) {
+        resizeBackground(size().toSize());
     }
 
     if (!m_animId && ~option->state & QStyle::State_Sunken) {
@@ -682,7 +699,7 @@ QSize AbstractTaskItem::layoutText(QTextLayout &layout, const QString &text,
     return QSize(widthUsed, height);
 }
 
-void AbstractTaskItem::drawTextLayout(QPainter *painter, const QTextLayout &layout, const QRect &rect) const
+void AbstractTaskItem::drawTextLayout(QPainter *painter, const QTextLayout &layout, const QRect &rect)
 {
     if (rect.width() < 1 || rect.height() < 1) {
         return;
@@ -750,13 +767,19 @@ void AbstractTaskItem::drawTextLayout(QPainter *painter, const QTextLayout &layo
         shadowColor = Qt::white;
     }
 
-    QImage shadow = pixmap.toImage();
-    Plasma::PaintUtils::shadowBlur(shadow, 2, shadowColor);
+    if (m_cachedShadow.isNull()) {
+        QImage shadow = pixmap.toImage();
+        Plasma::PaintUtils::shadowBlur(shadow, 2, shadowColor);
+        m_cachedShadow = QPixmap(shadow.size());
+        m_cachedShadow.fill(Qt::transparent);
+        QPainter buffPainter(&m_cachedShadow);
+        buffPainter.drawImage(QPoint(0,0), shadow);
+    }
 
     if (shadowColor == Qt::white) {
-        painter->drawImage(rect.topLeft(), shadow);
+        painter->drawPixmap(rect.topLeft(), m_cachedShadow);
     } else {
-        painter->drawImage(rect.topLeft() + QPoint(1,2), shadow);
+        painter->drawPixmap(rect.topLeft() + QPoint(1,2), m_cachedShadow);
     }
     painter->drawPixmap(rect.topLeft(), pixmap);
 }
