@@ -71,7 +71,7 @@ WebBrowser::WebBrowser(QObject *parent, const QVariantList &args)
 
     m_layout = 0;
     resize(500,500);
-    if (args.count() > 0) {
+    if (!args.isEmpty()) {
         m_url = KUrl(args.value(0).toString());
     }
     setPopupIcon("konqueror");
@@ -110,6 +110,7 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     m_browser = new Plasma::WebView(this);
     m_browser->setPreferredSize(400, 400);
     m_browser->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+    m_browser->setDragToScroll(cg.readEntry("DragToScroll", false));
 
 
     m_layout->addItem(m_browser);
@@ -158,13 +159,16 @@ QGraphicsWidget *WebBrowser::graphicsWidget()
     m_zoom->hide();
     m_zoom->setMaximumWidth(0);
     m_statusbarLayout->addItem(m_zoom);
-    connect(m_zoom, SIGNAL(valueChanged(int)), this, SLOT(zoom(int)));
 
     if (!m_url.isValid()) {
         m_url = KUrl(cg.readEntry("Url", "http://www.kde.org"));
         m_verticalScrollValue = cg.readEntry("VerticalScrollValue", 0);
         m_horizontalScrollValue = cg.readEntry("HorizontalScrollValue", 0);
+        int value = cg.readEntry("Zoom", 1);
+        m_zoom->setValue(value);
+        m_browser->mainFrame()->setZoomFactor((qreal)0.2 + ((qreal)value/(qreal)50));
     }
+    connect(m_zoom, SIGNAL(valueChanged(int)), this, SLOT(zoom(int)));
     m_browser->setUrl(m_url);
     m_browser->update();
 
@@ -350,8 +354,11 @@ void WebBrowser::dataUpdated( const QString &source, const Plasma::DataEngine::D
     // is only used to save the icon files
     if (source == m_historyCombo->currentText()) {
         QPixmap favicon(QPixmap::fromImage(data["Icon"].value<QImage>()));
-        m_historyCombo->nativeWidget()->setItemIcon(m_historyCombo->nativeWidget()->currentIndex(), QIcon(favicon));
-        setPopupIcon(QIcon(favicon));
+        if (!favicon.isNull()) {
+            m_historyCombo->nativeWidget()->setItemIcon(
+                                    m_historyCombo->nativeWidget()->currentIndex(), QIcon(favicon));
+            setPopupIcon(QIcon(favicon));
+        }
     }
 }
 
@@ -398,7 +405,7 @@ void WebBrowser::removeBookmark(const QModelIndex &index)
 
 void WebBrowser::removeBookmark()
 {
-    QModelIndexList list = m_bookmarkModel->match(m_bookmarkModel->index(0,0), BookmarkItem::UrlRole, m_url.prettyUrl());
+    const QModelIndexList list = m_bookmarkModel->match(m_bookmarkModel->index(0,0), BookmarkItem::UrlRole, m_url.prettyUrl());
 
     if (!list.isEmpty()) {
         removeBookmark(list.first());
@@ -438,7 +445,8 @@ void WebBrowser::bookmarkClicked(const QModelIndex &index)
 
 void WebBrowser::zoom(int value)
 {
-    m_browser->mainFrame()->setTextSizeMultiplier((qreal)0.2 + ((qreal)value/(qreal)50));
+    config().writeEntry("Zoom", value);
+    m_browser->mainFrame()->setZoomFactor((qreal)0.2 + ((qreal)value/(qreal)50));
 }
 
 void WebBrowser::loadProgress(int progress)
@@ -475,9 +483,17 @@ void WebBrowser::createConfigurationInterface(KConfigDialog *parent)
     parent->addPage(widget, i18n("General"), icon());
     connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
     connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
+    connect(ui.autoRefreshInterval, SIGNAL(valueChanged(int)), this, SLOT(updateSpinBoxSuffix(int)));
 
     ui.autoRefresh->setChecked(m_autoRefresh);
     ui.autoRefreshInterval->setValue(m_autoRefreshInterval);
+    updateSpinBoxSuffix(m_autoRefreshInterval);
+    ui.dragToScroll->setChecked(m_browser->dragToScroll());
+}
+
+void WebBrowser::updateSpinBoxSuffix(int interval)
+{
+    ui.autoRefreshInterval->setSuffix(i18np(" minute", " minutes", interval));
 }
 
 void WebBrowser::configAccepted()
@@ -489,6 +505,8 @@ void WebBrowser::configAccepted()
 
     cg.writeEntry("autoRefresh", m_autoRefresh);
     cg.writeEntry("autoRefreshInterval", m_autoRefreshInterval);
+    cg.writeEntry("DragToScroll", ui.dragToScroll->isChecked());
+    m_browser->setDragToScroll(ui.dragToScroll->isChecked());
 
     if (m_autoRefresh) {
         if (!m_autoRefreshTimer) {
