@@ -662,38 +662,6 @@ void Panel::saveState(KConfigGroup &config) const
 
 void Panel::themeUpdated()
 {
-    if (!layout()) {
-        return;
-    }
-
-    //if the theme is changed all the calculations needs to be done again
-    //and resize based on the change in the theme bordersize
-
-    qreal oldLeftWidth;
-    qreal newLeftWidth;
-    qreal oldTopHeight;
-    qreal newTopHeight;
-    qreal oldRightWidth;
-    qreal newRightWidth;
-    qreal oldBottomHeight;
-    qreal newBottomHeight;
-
-    layout()->getContentsMargins(&oldLeftWidth, &oldTopHeight, &oldRightWidth, &oldBottomHeight);
-    m_background->getMargins(newLeftWidth, newTopHeight, newRightWidth, newBottomHeight);
-
-    QSize newSize(size().width()-(oldLeftWidth - newLeftWidth)-(oldRightWidth - newRightWidth),
-           size().height()-(oldTopHeight - newTopHeight)-(oldBottomHeight - newBottomHeight));
-
-    resize(newSize);
-
-    if (formFactor() == Plasma::Vertical) {
-        setMaximumWidth(newSize.width());
-        setMinimumWidth(newSize.width());
-    } else {
-        setMaximumHeight(newSize.height());
-        setMinimumHeight(newSize.height());
-    }
-
     updateBorders(geometry().toRect(), true);
 }
 
