@@ -1644,7 +1644,7 @@ QString EnvCanadaIon::dewpoint(const QString& source)
 QString EnvCanadaIon::humidity(const QString& source)
 {
     if (!d->m_weatherData[source].humidity.isEmpty()) {
-        return QString("%1%").arg(d->m_weatherData[source].humidity);
+        return i18nc("Humidity in percent", "%1%", d->m_weatherData[source].humidity);
     }
     return "N/A";
 }
