@@ -1,2 +1,4 @@
 #! /usr/bin/env bash
+$EXTRACTRC *.ui >> rc.cpp
 $XGETTEXT *.cpp -o $podir/plasma-shells-common.pot
+rm -f rc.cpp
