@@ -97,20 +97,24 @@ void EditPage::fillUi()
     actionImportProfiles->setIcon(KIcon("document-import"));
     actionExportProfiles->setIcon(KIcon("document-export"));
 
-    idleCombo->addItem(KIcon("dialog-cancel"), i18n("Do nothing"), (int) None);
+    idleCombo->addItem(KIcon("dialog-cancel"), i18n("Do Nothing"), (int) None);
     idleCombo->addItem(KIcon("system-shutdown"), i18n("Shutdown"), (int) Shutdown);
     idleCombo->addItem(KIcon("system-lock-screen"), i18n("Lock Screen"), (int) Lock);
-    laptopClosedCombo->addItem(KIcon("dialog-cancel"), i18n("Do nothing"), (int) None);
+    idleCombo->addItem(KIcon("preferences-desktop-screensaver"), i18n("Turn Off Screen"), (int) TurnOffScreen);
+    laptopClosedCombo->addItem(KIcon("dialog-cancel"), i18n("Do Nothing"), (int) None);
     laptopClosedCombo->addItem(KIcon("system-shutdown"), i18n("Shutdown"), (int) Shutdown);
     laptopClosedCombo->addItem(KIcon("system-lock-screen"), i18n("Lock Screen"), (int) Lock);
-    sleepButtonCombo->addItem(KIcon("dialog-cancel"), i18n("Do nothing"), (int) None);
+    laptopClosedCombo->addItem(KIcon("preferences-desktop-screensaver"), i18n("Turn Off Screen"), (int) TurnOffScreen);
+    sleepButtonCombo->addItem(KIcon("dialog-cancel"), i18n("Do Nothing"), (int) None);
     sleepButtonCombo->addItem(KIcon("system-shutdown"), i18n("Shutdown"), (int) Shutdown);
     sleepButtonCombo->addItem(KIcon("system-lock-screen"), i18n("Lock Screen"), (int) Lock);
-    sleepButtonCombo->addItem(KIcon("system-log-out"), i18n("Prompt Log out dialog"), (int) ShutdownDialog);
-    powerButtonCombo->addItem(KIcon("dialog-cancel"), i18n("Do nothing"), (int) None);
+    sleepButtonCombo->addItem(KIcon("preferences-desktop-screensaver"), i18n("Turn Off Screen"), (int) TurnOffScreen);
+    sleepButtonCombo->addItem(KIcon("system-log-out"), i18n("Prompt Log Out dialog"), (int) ShutdownDialog);
+    powerButtonCombo->addItem(KIcon("dialog-cancel"), i18n("Do Nothing"), (int) None);
     powerButtonCombo->addItem(KIcon("system-shutdown"), i18n("Shutdown"), (int) Shutdown);
     powerButtonCombo->addItem(KIcon("system-lock-screen"), i18n("Lock Screen"), (int) Lock);
-    powerButtonCombo->addItem(KIcon("system-log-out"), i18n("Prompt Log out dialog"), (int) ShutdownDialog);
+    powerButtonCombo->addItem(KIcon("preferences-desktop-screensaver"), i18n("Turn Off Screen"), (int) TurnOffScreen);
+    powerButtonCombo->addItem(KIcon("system-log-out"), i18n("Prompt Log Out dialog"), (int) ShutdownDialog);
 
     Solid::Control::PowerManager::SuspendMethods methods = Solid::Control::PowerManager::supportedSuspendMethods();
 
@@ -127,10 +131,10 @@ void EditPage::fillUi()
     }
 
     if (methods & Solid::Control::PowerManager::ToRam) {
-        idleCombo->addItem(KIcon("system-suspend"), i18n("Suspend to Ram"), (int) S2Ram);
-        laptopClosedCombo->addItem(KIcon("system-suspend"), i18n("Suspend to Ram"), (int) S2Ram);
-        sleepButtonCombo->addItem(KIcon("system-suspend"), i18n("Suspend to Ram"), (int) S2Ram);
-        powerButtonCombo->addItem(KIcon("system-suspend"), i18n("Suspend to Ram"), (int) S2Ram);
+        idleCombo->addItem(KIcon("system-suspend"), i18n("Suspend to RAM"), (int) S2Ram);
+        laptopClosedCombo->addItem(KIcon("system-suspend"), i18n("Suspend to RAM"), (int) S2Ram);
+        sleepButtonCombo->addItem(KIcon("system-suspend"), i18n("Suspend to RAM"), (int) S2Ram);
+        powerButtonCombo->addItem(KIcon("system-suspend"), i18n("Suspend to RAM"), (int) S2Ram);
     }
 
     if (methods & Solid::Control::PowerManager::Standby) {
@@ -195,11 +199,13 @@ void EditPage::fillUi()
     toolBox->setItemIcon(1, KIcon("video-display"));
     toolBox->setItemIcon(2, KIcon("cpu"));
 
+#if 0 // Re-enable when / if we have permission to use official logo
     DPMSLabel->setUrl("http://www.energystar.gov");
     DPMSLabel->setPixmap(QPixmap(KStandardDirs::locate("data", "kcontrol/pics/energybig.png")));
     DPMSLabel->setTipText(i18n("Learn more about the Energy Star program"));
     DPMSLabel->setUseTips(true);
     connect(DPMSLabel, SIGNAL(leftClickedUrl(const QString&)), SLOT(openUrl(const QString &)));
+#endif
 
 #ifndef HAVE_DPMS
     DPMSEnable->setEnabled(false);
@@ -318,7 +324,7 @@ void EditPage::loadProfile()
     idleCombo->setCurrentIndex(idleCombo->findData(group->readEntry("idleAction").toInt()));
     freqCombo->setCurrentIndex(freqCombo->findData(group->readEntry("cpuPolicy").toInt()));
     schemeCombo->setCurrentIndex(schemeCombo->findText(group->readEntry("scheme")));
-    scriptRequester->setPath(group->readEntry("scriptpath"));
+    scriptRequester->setUrl(KUrl::fromPath(group->readEntry("scriptpath")));
 
     laptopClosedCombo->setCurrentIndex(laptopClosedCombo->findData(group->readEntry("lidAction").toInt()));
     sleepButtonCombo->setCurrentIndex(sleepButtonCombo->findData(group->readEntry("sleepButtonAction").toInt()));
@@ -398,7 +404,7 @@ void EditPage::saveProfile(const QString &p)
 
     QList<int> list;
 
-    for (int i = 0;i < CPUListLayout->count();++i) {
+    for (int i = 0; i < CPUListLayout->count(); ++i) {
         QCheckBox *box = qobject_cast<QCheckBox*> (CPUListLayout->itemAt(i)->widget());
 
         if (!box)
@@ -491,7 +497,7 @@ void EditPage::createProfile()
 
     ibt->setIconSize(KIconLoader::SizeSmall);
 
-    lb->setText(i18n("Please enter a name for the new profile"));
+    lb->setText(i18n("Please enter a name for the new profile:"));
 
     lb->setToolTip(i18n("The name for the new profile"));
     lb->setWhatsThis(i18n("Enter here the name for the profile you are creating"));
@@ -545,7 +551,7 @@ void EditPage::editProfile()
 
     ibt->setIconSize(KIconLoader::SizeSmall);
 
-    lb->setText(i18n("Please enter a name for this profile"));
+    lb->setText(i18n("Please enter a name for this profile:"));
 
     lb->setToolTip(i18n("The name for the new profile"));
     lb->setWhatsThis(i18n("Enter here the name for the profile you are creating"));
@@ -579,7 +585,7 @@ void EditPage::editProfile()
 void EditPage::importProfiles()
 {
     QString fileName = KFileDialog::getOpenFileName(KUrl(), "*.powerdevilprofiles|PowerDevil Profiles "
-                       "(*.powerdevilprofiles)", this, i18n("Import PowerDevil profiles"));
+                       "(*.powerdevilprofiles)", this, i18n("Import PowerDevil Profiles"));
 
     if (fileName.isEmpty()) {
         return;
@@ -604,7 +610,7 @@ void EditPage::importProfiles()
 void EditPage::exportProfiles()
 {
     QString fileName = KFileDialog::getSaveFileName(KUrl(), "*.powerdevilprofiles|PowerDevil Profiles "
-                       "(*.powerdevilprofiles)", this, i18n("Export PowerDevil profiles"));
+                       "(*.powerdevilprofiles)", this, i18n("Export PowerDevil Profiles"));
 
     if (fileName.isEmpty()) {
         return;
